/*  COSMIX: general n_body simulator, version 0.4 Beta
    Copyright (C) 2001 Shan Mignot

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <gtk/gtk.h>
#include <GL/gl.h>
#include <GL/glu.h>
#include <gtkgl/gtkglarea.h>
#include <libxml/tree.h>
#include <libxml/xmlmemory.h>

#include "define.h"
#include "functions.h"
#include "text.h"
#include "dop853.h"

gint window_destroy_event (GtkWidget *widget, GdkEvent *event, gpointer id)
{
  gtk_widget_destroy (global_widgets[GPOINTER_TO_INT (id)].widget);
  global_widgets[GPOINTER_TO_INT (id)].exists = FALSE;
  return (TRUE);
}

void window_destroy_signal (GtkWidget *widget,
			    gpointer id)
{
  gtk_widget_destroy (global_widgets[GPOINTER_TO_INT (id)].widget);
  global_widgets[GPOINTER_TO_INT (id)].exists = FALSE;
}

void destroy_structure (gpointer *data)
{
  Structure *structure;
  
  structure = (Structure*) data;
  g_free (structure->structure);
  g_free (structure);
}

void destroy_computation_structures (void)
{
  if (input)
    {
      g_free (input->controls);
      g_free (((NPhyInput*)input->phy)->masses);
      g_free (((NPhyInput*)input->phy)->dimensions);
      g_free (input->phy);
      g_free (((NKiInput*)input->ki)->positions);
      g_free (input->ki);
      g_free (input);
      input = NULL;
    }
  if (output)
    {
      int i;

      for (i = 0; i < output->n_body; i++)
	{
	  g_free (((NObjectOutput*)output->objects)[i].x);
	  g_free (((NObjectOutput*)output->objects)[i].y);
	  g_free (((NObjectOutput*)output->objects)[i].z);
	  g_free (((NObjectOutput*)output->objects)[i].vx);
	  g_free (((NObjectOutput*)output->objects)[i].vy);
	  g_free (((NObjectOutput*)output->objects)[i].vz);
	}
      g_free (output->objects);
      g_free (output);
      output = NULL;
    }
}
/* -- Main menu callback functions -- */

/* The file dialog box should be setup by this function,
   the only parameters are its name and the callback for
   OK/
   void save_dialog_setup (const char title, 
*/

/* File menu */
/* reset everything */
void project_new (GtkWidget *widget, gpointer data)
{
  int i;

  system_new (NULL, 1, widget);
  destroy_computation_structures ();
  if (grp_display_list)
    {
      glDeleteLists (grp_display_list->first_list, grp_display_list->range);
      grp_display_list = NULL;
    }
  render_def_mesh ();
  render_grp_mesh ();
  for (i = 0; i < 4; i++)
    {
      redraw_viewport (global_widgets[DEF_GL_X + i].widget);
      redraw_viewport (global_widgets[GRP_GL_X + i].widget);
    }
  tbl_new (NULL, NULL);
  script_new (widget, data);
}

void system_new (gpointer data,
		 guint callback_action,
		 GtkWidget *widget)
{
  GtkCList *system_list = GTK_CLIST (global_widgets[SYSTEM_TREE].widget);

  gtk_clist_freeze (system_list);
  gtk_ctree_post_recursive (GTK_CTREE (system_list), NULL,
			    GTK_CTREE_FUNC (gtk_ctree_remove_node),
			    NULL);
  gtk_clist_thaw (system_list);
}

/* delete the content of the tbl page (and free the allocated memory) */
void tbl_new (GtkWidget *widget, gpointer data)
{
  GtkCList *tbl = NULL;

  if (!global_widgets[TBL].exists)
    return;
  
  tbl = GTK_CLIST (global_widgets[TBL].widget);
  gtk_clist_clear (tbl);
}

/* delete the content of the script page */
void script_new (GtkWidget *widget, gpointer data)
{
  gtk_editable_delete_text (GTK_EDITABLE (global_widgets[SCRIPT].widget), 0, -1);
}

void project_load (GtkWidget *widget,
		   gpointer data)
{  
  SETUP_LOAD_SAVE_DIALOG (WINDOW_LOAD_PROJECT, PROJECT_DIRECTORY,
			  FILE_EXTENSION_PROJECT, parse_project_file);
}

void system_load (GtkWidget *widget, gpointer data)
{
  SETUP_LOAD_SAVE_DIALOG (WINDOW_LOAD_SYSTEM, SYSTEM_DIRECTORY,
			  FILE_EXTENSION_SYSTEM, parse_physicalsystem_file);
}

void script_load (GtkWidget *widget, gpointer data)
{
  SETUP_LOAD_SAVE_DIALOG (WINDOW_LOAD_SCRIPT, SYSTEM_DIRECTORY,
			  FILE_EXTENSION_SYSTEM, parse_script_file);
}

void objects_load (GtkWidget *widget, gpointer data)
{
}

void cnsts_load (GtkWidget *widget, gpointer data)
{
}

void reload (GtkWidget *widget, gpointer data)
{
}

/* Restart COSMIX */
void reset (GtkWidget *widget, gpointer data)
{
  project_new (NULL, NULL);
}

void save (GtkWidget *widget, gpointer data)
{
}

void project_save_as (GtkWidget *widget, gpointer data)
{
}

void system_save_as (GtkWidget *widget, gpointer data)
{
  SETUP_LOAD_SAVE_DIALOG (WINDOW_SAVE_SYSTEM, SYSTEM_DIRECTORY,
			  FILE_EXTENSION_SYSTEM, system_save_as_xml);
}

void script_save_as (GtkWidget *widget, gpointer data)
{
  SETUP_LOAD_SAVE_DIALOG (WINDOW_SAVE_SCRIPT, SYSTEM_DIRECTORY,
			  FILE_EXTENSION_SYSTEM, script_save_as_system);
}

/* Write results to file */
void tbl_save_as (GtkWidget *widget, gpointer data)
{
  SETUP_LOAD_SAVE_DIALOG (WINDOW_SAVE_RESULTS, RESULTS_DIRECTORY,
			  FILE_EXTENSION_RESULTS, results_save_as_text);
}

void animation_save_as (GtkWidget *widget, gpointer data)
{
}

void objects_save_as (GtkWidget *widget, gpointer data)
{
}

void cnsts_save_as (GtkWidget *widget, gpointer data)
{
}

void properties (GtkWidget *widget, gpointer data)
{
}

void quit (GtkWidget *widget,
	   gpointer data)
{
  GtkWidget *quit_confirm_message = gtk_label_new (QUIT_MESSAGE);
  GtkWidget *ok_button  = gtk_button_new_with_label (OK);
  GtkWidget *cancel_button = gtk_button_new_with_label (CANCEL);
  gint id = QUIT_DIALOG;
  
  /* !!!!!!!!!! Don't forget to free the memory when exiting !!!!!!!!!! */
  if (!global_widgets[id].exists)
    {
      SETUP_DIALOG(global_widgets[id].widget, WINDOW_QUIT, GINT_TO_POINTER (id)); 
      
      gtk_box_pack_start (GTK_BOX (GTK_DIALOG (global_widgets[id].widget)
				   -> vbox), quit_confirm_message,
			  FALSE, FALSE, NO_PADDING);
      
      gtk_signal_connect (GTK_OBJECT (ok_button), "clicked",
			  GTK_SIGNAL_FUNC (gtk_main_quit), NULL);
      gtk_box_pack_start (GTK_BOX (GTK_DIALOG (global_widgets[id].widget
					       )-> action_area),
			  ok_button, FALSE, FALSE, NO_PADDING);
      GTK_WIDGET_SET_FLAGS (ok_button, GTK_CAN_DEFAULT);
      gtk_widget_grab_default (ok_button);
      
      gtk_signal_connect (GTK_OBJECT (cancel_button),
			  "clicked",
			  GTK_SIGNAL_FUNC (window_destroy_signal),
			  GINT_TO_POINTER (id));
      gtk_box_pack_start (GTK_BOX (GTK_DIALOG (global_widgets[id].widget)
				   -> action_area),
			  cancel_button, FALSE, FALSE, NO_PADDING);
      GTK_WIDGET_SET_FLAGS (cancel_button, GTK_CAN_DEFAULT);
      gtk_widget_show_all (global_widgets[id].widget);
    }
}

void window_show (GtkWidget *widget,
		  gpointer data)
{
  gtk_widget_show (global_widgets[GPOINTER_TO_INT (data)].widget);
}

void getting_started (GtkWidget *widget,
		      gpointer data)
{
  gchar *text;
  
  text = g_strdup_printf (GETTING_STARTED_MESSAGE);
  SETUP_MESSAGE_DIALOG (WINDOW_GETTING_STARTED, text);
}

void about (GtkWidget *widget,
	    gpointer data)
{
  gchar *text;
  
  text = g_strdup_printf (ABOUT_MESSAGE);
  SETUP_MESSAGE_DIALOG (WINDOW_ABOUT, text);
}

/* -- Control Panel callback functions -- */

void tree_utils (GtkCList *system_list,
		 gint column,
		 gpointer data)
{
  gint id =  TEXT_ENTRY;
  GtkWidget *vbox = gtk_vbox_new (FALSE, NO_PADDING);
  GtkWidget *hbox = gtk_hbox_new (TRUE, NO_PADDING);  
  GtkWidget *ok_button = gtk_button_new_with_label (OK);
  GtkWidget *cancel_button = gtk_button_new_with_label (CANCEL);
  GtkWidget *text_entry = gtk_entry_new ();
  GtkCTreeNode *selected_node = NULL;
  TwoWidgets *widgets = NULL;
  PhyObject *object = NULL;


  widgets = g_new0 (TwoWidgets, 1);
  widgets->widget1 = GTK_WIDGET (system_list);
  widgets->widget2 = text_entry;

  /* If there is no selection don't do anything */
  if (system_list->selection == NULL) return;

  switch (column)
    {
    case 2:
      /* Changing the status (active/inactive) of the object */
      selected_node = GTK_CTREE_NODE (system_list->selection->data);
      if (GTK_CTREE_ROW (selected_node)->is_leaf)
	selected_node = GTK_CTREE_NODE (GTK_CTREE_ROW (selected_node)->parent);
      object = (PhyObject*) ((Structure*) 
			  (gtk_ctree_node_get_row_data
			   (GTK_CTREE (system_list), selected_node)))->structure;
      if (object->active)
	{
	  object->active = FALSE;
	  gtk_ctree_node_set_pixmap
	    (GTK_CTREE (system_list), selected_node, 2, 
	     new_pixmap ((global_widgets[CONTROL_PANEL].widget)->window, 
			 &(global_widgets[CONTROL_PANEL].widget)->style->
			 bg[GTK_STATE_NORMAL], ICON_NOT_OK),
	     new_bitmap ((global_widgets[CONTROL_PANEL].widget)->window, 
			 &(global_widgets[CONTROL_PANEL].widget)->style->
			 bg[GTK_STATE_NORMAL], ICON_NOT_OK));
	}
      else
	{
	 object->active = TRUE;
	 gtk_ctree_node_set_pixmap
	   (GTK_CTREE (system_list), selected_node, 2, 
	    new_pixmap ((global_widgets[CONTROL_PANEL].widget)->window, 
			&(global_widgets[CONTROL_PANEL].widget)->style->
			bg[GTK_STATE_NORMAL], ICON_OK),
	    new_bitmap ((global_widgets[CONTROL_PANEL].widget)->window, 
			&(global_widgets[CONTROL_PANEL].widget)->style->
			bg[GTK_STATE_NORMAL], ICON_OK));
	}
      break;
    case 3:
      /* Changing the name of the object */
      if (global_widgets[id].exists)
	gtk_widget_destroy (global_widgets[id].widget);
      ENTRY_WINDOW(global_widgets[id].widget, NAME, GINT_TO_POINTER (id));
      global_widgets[id].exists = TRUE;
      gtk_container_add (GTK_CONTAINER (global_widgets[id].widget), vbox);
      gtk_box_pack_start (GTK_BOX (vbox), text_entry,
			  FALSE, FALSE, NO_PADDING);
      gtk_box_pack_start (GTK_BOX (vbox), hbox,
			  FALSE, FALSE, NO_PADDING);
      gtk_box_pack_start (GTK_BOX (hbox), ok_button,
			  FALSE, FALSE, NO_PADDING);
      gtk_box_pack_start (GTK_BOX (hbox), cancel_button,
			  FALSE, FALSE, NO_PADDING);
      gtk_widget_grab_focus (text_entry);
      gtk_signal_connect (GTK_OBJECT (cancel_button), "clicked",
			  GTK_SIGNAL_FUNC (window_destroy_signal), 
			  GINT_TO_POINTER (id));      
      gtk_signal_connect (GTK_OBJECT (ok_button), "clicked", 
			  GTK_SIGNAL_FUNC (edit_name), widgets);
      gtk_signal_connect (GTK_OBJECT (text_entry), "activate",
			  GTK_SIGNAL_FUNC (edit_name), widgets);
      gtk_widget_show_all (global_widgets[id].widget);
      break;
    case 4:
      /* Changing the comment of the selection */
      if (global_widgets[id].exists)
       	  gtk_widget_destroy (global_widgets[id].widget);
      ENTRY_WINDOW(global_widgets[id].widget, NAME, GINT_TO_POINTER (id));
      global_widgets[id].exists = TRUE;
      gtk_container_add (GTK_CONTAINER (global_widgets[id].widget), vbox);
      gtk_box_pack_start (GTK_BOX (vbox), text_entry,
			  FALSE, FALSE, NO_PADDING);
      gtk_box_pack_start (GTK_BOX (vbox), hbox,
			  FALSE, FALSE, NO_PADDING);
      gtk_box_pack_start (GTK_BOX (hbox), ok_button,
			  FALSE, FALSE, NO_PADDING);
      gtk_box_pack_start (GTK_BOX (hbox), cancel_button,
			  FALSE, FALSE, NO_PADDING);
      gtk_widget_grab_focus (text_entry);
      gtk_signal_connect (GTK_OBJECT (cancel_button), "clicked",
			  GTK_SIGNAL_FUNC (window_destroy_signal), 
			  GINT_TO_POINTER (id));      
      gtk_signal_connect (GTK_OBJECT (ok_button), "clicked", 
			  GTK_SIGNAL_FUNC (edit_comment), widgets);
      gtk_signal_connect (GTK_OBJECT (text_entry), "activate",
			  GTK_SIGNAL_FUNC (edit_comment), widgets);
      gtk_widget_show_all (global_widgets[id].widget);
      break;
    }
}

void edit_name (GtkWidget *widget,
		gpointer *data)
{
  TwoWidgets *widgets;
  GtkCTreeNode *selected_node = NULL;
  gchar *name = NULL;
  PhyObject *object = NULL;

  widgets = (TwoWidgets*) data;
  selected_node = GTK_CLIST (widgets->widget1)->selection->data;
  if (GTK_CTREE_ROW (selected_node)->is_leaf)
    selected_node = GTK_CTREE_ROW (selected_node)->parent;
  name = gtk_editable_get_chars (GTK_EDITABLE (widgets->widget2), 0, -1);
  gtk_ctree_node_set_text (GTK_CTREE (widgets->widget1),
			   GTK_CTREE_NODE (selected_node), 3, name);
  object = (PhyObject*)
    ((Structure*) gtk_ctree_node_get_row_data 
     (GTK_CTREE (widgets->widget1), GTK_CTREE_NODE (selected_node)))->structure;
  object->name = name;
  window_destroy_signal (widget, GINT_TO_POINTER (TEXT_ENTRY));
  g_free (widgets);
}

void edit_comment (GtkWidget *widget,
		   gpointer *data)
{
  TwoWidgets *widgets;
  GtkCTreeNode *selected_node = NULL;
  gchar *comment = NULL;
  Structure *structure = NULL;

  widgets = (TwoWidgets*) data;
  selected_node = GTK_CLIST (widgets->widget1)->selection->data;
  comment = gtk_editable_get_chars (GTK_EDITABLE (widgets->widget2), 0, -1);
  gtk_ctree_node_set_text (GTK_CTREE (widgets->widget1),
			   GTK_CTREE_NODE (selected_node), 4, comment);
  structure = (Structure*)
    gtk_ctree_node_get_row_data (GTK_CTREE (widgets->widget1),
				 GTK_CTREE_NODE (selected_node));
  switch (structure->type)
    {
    case OBJECT:
      ((PhyObject*) structure->structure)->comment = comment;
      break;
    case PHY:
      ((PtObjectPhy*) structure->structure)->comment = comment;
      break;
    case KI:
      ((PtObjectKi*) structure->structure)->comment = comment;
      break;
    }
  window_destroy_signal (widget, GINT_TO_POINTER (TEXT_ENTRY));
  g_free (widgets);
}

gint tree_popup_menu (GtkWidget *system_list,
		      GdkEventButton *event,
		      gpointer data)
{
  GtkWidget *menu = NULL;
  GtkWidget *item = NULL;
  GtkItemFactory *menu_factory = NULL;
  GtkAccelGroup *menu_accel_group = gtk_accel_group_new ();
  GtkItemFactoryEntry menu_items[] =
  {
    { NEW,                 NULL,         NULL,              0,  "<Branch>"},
    { NEW_PT_OBJ,          NULL,         NULL,              0,  NULL},
    { COPY,                NULL,         copy_element,      0,  NULL},
    { EXPAND,              NULL,         expand_tree,    TRUE,  NULL},
    { COLLAPSE,            NULL,         expand_tree,   FALSE,   NULL},
    { DELETE,              NULL,         delete_element,    0,  NULL},
    { DELETE_ALL,          NULL,         system_new,        0,  NULL}
  };
 
  if (event->button == 3)
    {
   
      /* Build the menu */
      menu_factory = gtk_item_factory_new (GTK_TYPE_MENU, "<main>",
					   menu_accel_group);
      gtk_item_factory_create_items (menu_factory,
				     sizeof (menu_items) / sizeof (menu_items[0]),
				     menu_items, NULL);
      menu = gtk_item_factory_get_widget (menu_factory, "<main>");
      if (GTK_CLIST (system_list)->selection == NULL)
	{
	  gtk_widget_set_sensitive 
	    (GTK_WIDGET (gtk_item_factory_get_item (menu_factory, COPY)), FALSE);
	  gtk_widget_set_sensitive 
	    (GTK_WIDGET (gtk_item_factory_get_item (menu_factory, DELETE)), FALSE);
	}
      /* For standard callbacks, set them up manually */
      item = gtk_item_factory_get_widget (menu_factory, NEW_PT_OBJ);
      gtk_signal_connect (GTK_OBJECT (item), "activate", 
			  GTK_SIGNAL_FUNC (pt_obj_new), NULL);
      /* Display the menu */
      gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL, 
		      event->button, event->time);
      return (TRUE);
    }
  return (FALSE);
}

void copy_element (gpointer data,
		   guint callback_action,
		   GtkWidget *widget)
{
  GtkCTreeNode *reference_node = NULL;
  GtkCTreeNode *new_node = NULL;
  GtkCTree *system_tree = GTK_CTREE (global_widgets[SYSTEM_TREE].widget);
  GtkCList *system_list = GTK_CLIST (system_tree);
  PhyObject *ref_obj = NULL, *new_obj = NULL;
  PtObjectPhy *ref_obj_phy = NULL, *new_obj_phy = NULL;
  PtObjectKi *ref_obj_ki = NULL, *new_obj_ki = NULL;
  gint i;

  gtk_clist_freeze (system_list);
  reference_node = GTK_CTREE_NODE (system_list->selection->data);
  if (GTK_CTREE_ROW (reference_node)->is_leaf)
    reference_node = GTK_CTREE_NODE (GTK_CTREE_ROW (reference_node)->parent);
  pt_obj_new (NULL, NULL);
  new_node = GTK_CTREE_NODE (system_list->selection->data);

  /* Now make it inactive because the objects have identical locations
     and copy the fields of the reference node */
  /* PhyObject */
  ref_obj = (PhyObject*)
    ((Structure*) gtk_ctree_node_get_row_data (system_tree,
					       reference_node))->structure;
  new_obj = (PhyObject*)
    ((Structure*) gtk_ctree_node_get_row_data (system_tree,
					       new_node))->structure;
  new_obj->object_type = ref_obj->object_type;
  new_obj->active = FALSE;
  gtk_ctree_node_set_pixmap
    (system_tree, new_node, 2, 
     new_pixmap ((global_widgets[CONTROL_PANEL].widget)->window, 
		 &(global_widgets[CONTROL_PANEL].widget)->style->
		 bg[GTK_STATE_NORMAL], ICON_NOT_OK),
     new_bitmap ((global_widgets[CONTROL_PANEL].widget)->window, 
		 &(global_widgets[CONTROL_PANEL].widget)->style->
		 bg[GTK_STATE_NORMAL], ICON_NOT_OK));
  new_obj->ok = ref_obj->ok;
  if (new_obj->ok)
    gtk_ctree_node_set_pixmap
      (system_tree, new_node, 1, 
       new_pixmap ((global_widgets[CONTROL_PANEL].widget)->window, 
		   &(global_widgets[CONTROL_PANEL].widget)->style->
		   bg[GTK_STATE_NORMAL], ICON_CHECK_OK),
       new_bitmap ((global_widgets[CONTROL_PANEL].widget)->window, 
		   &(global_widgets[CONTROL_PANEL].widget)->style->
		   bg[GTK_STATE_NORMAL], ICON_CHECK_OK));
  /* PtObjectPhy */
  reference_node = GTK_CTREE_NODE (GTK_CTREE_ROW (reference_node)->children);
  new_node = GTK_CTREE_NODE (GTK_CTREE_ROW (new_node)->children);
  ref_obj_phy = (PtObjectPhy*)
    ((Structure*) gtk_ctree_node_get_row_data (system_tree,
					      reference_node))->structure;
  new_obj_phy = (PtObjectPhy*)
    ((Structure*) gtk_ctree_node_get_row_data (system_tree,
					       new_node))->structure;
  new_obj_phy->mass = ref_obj_phy->mass;
  new_obj_phy->mass_ok = ref_obj_phy->mass_ok;
  new_obj_phy->dimension = ref_obj_phy->dimension;
  new_obj_phy->dimension_exists = ref_obj_phy->dimension_exists;
  new_obj_phy->ok = ref_obj_phy->ok;
  if (new_obj_phy->ok)
    gtk_ctree_node_set_pixmap
      (system_tree, new_node, 1, 
       new_pixmap ((global_widgets[CONTROL_PANEL].widget)->window, 
		   &(global_widgets[CONTROL_PANEL].widget)->style->
		   bg[GTK_STATE_NORMAL], ICON_CHECK_OK),
       new_bitmap ((global_widgets[CONTROL_PANEL].widget)->window, 
		   &(global_widgets[CONTROL_PANEL].widget)->style->
		   bg[GTK_STATE_NORMAL], ICON_CHECK_OK));
  /* PtObjectKi */
  reference_node = GTK_CTREE_NODE (GTK_CTREE_ROW (reference_node)->sibling);
  new_node = GTK_CTREE_NODE (GTK_CTREE_ROW (new_node)->sibling);
  ref_obj_ki = (PtObjectKi*)
    ((Structure*) gtk_ctree_node_get_row_data (system_tree,
					       reference_node))->structure;
  new_obj_ki = (PtObjectKi*)
    ((Structure*) gtk_ctree_node_get_row_data (system_tree,
					       new_node))->structure;
  for (i = 0; i < 3; i++)
    {
      new_obj_ki->positions[i] = ref_obj_ki->positions[i];
      new_obj_ki->speeds[i] = ref_obj_ki->speeds[i];
    }
  new_obj_ki->position_x_ok = ref_obj_ki->position_x_ok;
  new_obj_ki->position_y_ok = ref_obj_ki->position_y_ok;
  new_obj_ki->position_z_ok = ref_obj_ki->position_z_ok;
  new_obj_ki->speed_x_ok = ref_obj_ki->speed_x_ok;
  new_obj_ki->speed_y_ok = ref_obj_ki->speed_y_ok;
  new_obj_ki->speed_z_ok = ref_obj_ki->speed_z_ok;
  new_obj_ki->ok = ref_obj_ki->ok;
  if (new_obj_ki->ok)
    gtk_ctree_node_set_pixmap
      (system_tree, new_node, 1, 
       new_pixmap ((global_widgets[CONTROL_PANEL].widget)->window, 
		   &(global_widgets[CONTROL_PANEL].widget)->style->
		   bg[GTK_STATE_NORMAL], ICON_CHECK_OK),
       new_bitmap ((global_widgets[CONTROL_PANEL].widget)->window, 
		   &(global_widgets[CONTROL_PANEL].widget)->style->
		   bg[GTK_STATE_NORMAL], ICON_CHECK_OK));
  gtk_clist_thaw (system_list);
}

void expand_tree (gpointer data,
		  guint callback_action,
		  GtkWidget *widget)
{
  GtkCTreeNode *node;
  GtkCTree *system_tree = GTK_CTREE (global_widgets[SYSTEM_TREE].widget);

  gtk_clist_freeze (GTK_CLIST (system_tree));
  node = gtk_ctree_node_nth (system_tree, 0);
  if (!node) return;

  while (node)
    {
      if (callback_action == TRUE)
	gtk_ctree_expand (system_tree, node);
      else
	gtk_ctree_collapse (system_tree, node);
      node = GTK_CTREE_NODE_NEXT (node);
    }
  if (callback_action == TRUE)
    {
      if (GTK_CLIST (system_tree)->selection)
	gtk_ctree_unselect 
	  (system_tree, GTK_CTREE_NODE (GTK_CLIST (system_tree)->selection->data));
    } 
  else 
    gtk_ctree_select (system_tree, GTK_CTREE_NODE 
		      (GTK_CTREE_ROW (GTK_CLIST (system_tree)->selection->data)
		       ->parent));
  gtk_clist_thaw (GTK_CLIST (system_tree));
}

void delete_element (gpointer data,
		     guint callback_action,
		     GtkWidget *widget)
{
  GtkCList *system_list = GTK_CLIST (global_widgets[SYSTEM_TREE].widget);
  GtkCTreeNode *selected_node;

  gtk_clist_freeze (system_list);
  selected_node = system_list->selection->data;
  if (GTK_CTREE_ROW (selected_node)->is_leaf)
    selected_node = GTK_CTREE_ROW (selected_node)->parent;
  gtk_ctree_post_recursive (GTK_CTREE (system_list), selected_node,
			    GTK_CTREE_FUNC (gtk_ctree_remove_node),
			    NULL);
  gtk_clist_thaw (system_list);
}

void pt_obj_new (GtkWidget *widget,
		 gpointer data)
{
  gint i;
  GtkCTree *system_tree = GTK_CTREE (global_widgets[SYSTEM_TREE].widget);
  GtkCTreeNode *inserted_node;
  GtkCTreeNode *child_node[4];
  gchar *no_text[] = {NO_TEXT, NULL, NULL, NULL, NULL};
  Structure *structure_object = NULL, *structure_phy = NULL, *structure_ki = NULL;
  PhyObject *object = NULL;
  PtObjectPhy *phy = NULL;
  PtObjectKi *ki = NULL;

  gtk_clist_freeze (GTK_CLIST (system_tree));
  /* Object level */
  inserted_node = gtk_ctree_insert_node
    (system_tree, NULL, NULL, no_text, NO_PADDING, 
     new_pixmap ((global_widgets[CONTROL_PANEL].widget)->window, 
		 &(global_widgets[CONTROL_PANEL].widget)->style->
		 bg[GTK_STATE_NORMAL], ICON_PT_OBJ),
     new_bitmap ((global_widgets[CONTROL_PANEL].widget)->window, 
		 &(global_widgets[CONTROL_PANEL].widget)->style->
		 bg[GTK_STATE_NORMAL], ICON_PT_OBJ),
     new_pixmap ((global_widgets[CONTROL_PANEL].widget)->window, 
		 &(global_widgets[CONTROL_PANEL].widget)->style->
		 bg[GTK_STATE_NORMAL], ICON_PT_OBJ),
     new_bitmap ((global_widgets[CONTROL_PANEL].widget)->window, 
		 &(global_widgets[CONTROL_PANEL].widget)->style->
		 bg[GTK_STATE_NORMAL], ICON_PT_OBJ),
     FALSE, TRUE);
  gtk_ctree_node_set_pixmap
    (system_tree, inserted_node, 1, 
     new_pixmap ((global_widgets[CONTROL_PANEL].widget)->window, 
		 &(global_widgets[CONTROL_PANEL].widget)->style->
		 bg[GTK_STATE_NORMAL], ICON_CHECK_NOT_OK),
     new_bitmap ((global_widgets[CONTROL_PANEL].widget)->window, 
		 &(global_widgets[CONTROL_PANEL].widget)->style->
		 bg[GTK_STATE_NORMAL], ICON_CHECK_NOT_OK));
  gtk_ctree_node_set_pixmap
    (GTK_CTREE (system_tree), inserted_node, 2, 
     new_pixmap ((global_widgets[CONTROL_PANEL].widget)->window, 
		 &(global_widgets[CONTROL_PANEL].widget)->style->
		 bg[GTK_STATE_NORMAL], ICON_OK),
     new_bitmap ((global_widgets[CONTROL_PANEL].widget)->window, 
		 &(global_widgets[CONTROL_PANEL].widget)->style->
		 bg[GTK_STATE_NORMAL], ICON_OK));
  
  /* Default dependencies */
  child_node[0] =  gtk_ctree_insert_node 
    (system_tree, inserted_node, NULL, no_text, NO_PADDING,
     new_pixmap ((global_widgets[CONTROL_PANEL].widget)->window, 
		 &(global_widgets[CONTROL_PANEL].widget)->style->
		 bg[GTK_STATE_NORMAL], ICON_PHI),
     new_bitmap ((global_widgets[CONTROL_PANEL].widget)->window, 
		 &(global_widgets[CONTROL_PANEL].widget)->style->
		 bg[GTK_STATE_NORMAL], ICON_PHI),
     new_pixmap ((global_widgets[CONTROL_PANEL].widget)->window, 
		 &(global_widgets[CONTROL_PANEL].widget)->style->
		 bg[GTK_STATE_NORMAL], ICON_PHI),
     new_bitmap ((global_widgets[CONTROL_PANEL].widget)->window, 
		 &(global_widgets[CONTROL_PANEL].widget)->style->
		 bg[GTK_STATE_NORMAL], ICON_PHI),
     TRUE, FALSE);
  child_node[1] = gtk_ctree_insert_node 
    (system_tree, inserted_node, NULL, no_text, NO_PADDING,
     new_pixmap ((global_widgets[CONTROL_PANEL].widget)->window, 
		 &(global_widgets[CONTROL_PANEL].widget)->style->
		 bg[GTK_STATE_NORMAL], ICON_CHI),
     new_bitmap ((global_widgets[CONTROL_PANEL].widget)->window, 
		 &(global_widgets[CONTROL_PANEL].widget)->style->
		 bg[GTK_STATE_NORMAL], ICON_CHI),
     new_pixmap ((global_widgets[CONTROL_PANEL].widget)->window, 
		 &(global_widgets[CONTROL_PANEL].widget)->style->
		 bg[GTK_STATE_NORMAL], ICON_CHI),
     new_bitmap ((global_widgets[CONTROL_PANEL].widget)->window, 
		 &(global_widgets[CONTROL_PANEL].widget)->style->
		 bg[GTK_STATE_NORMAL], ICON_CHI),
     TRUE, FALSE);
  child_node[2] = gtk_ctree_insert_node 
    (system_tree, inserted_node, NULL, no_text, NO_PADDING,
     new_pixmap ((global_widgets[CONTROL_PANEL].widget)->window, 
		 &(global_widgets[CONTROL_PANEL].widget)->style->
		 bg[GTK_STATE_NORMAL], ICON_GRP),
     new_bitmap ((global_widgets[CONTROL_PANEL].widget)->window, 
		 &(global_widgets[CONTROL_PANEL].widget)->style->
		 bg[GTK_STATE_NORMAL], ICON_GRP),
     new_pixmap ((global_widgets[CONTROL_PANEL].widget)->window, 
		 &(global_widgets[CONTROL_PANEL].widget)->style->
		 bg[GTK_STATE_NORMAL], ICON_GRP),
     new_bitmap ((global_widgets[CONTROL_PANEL].widget)->window, 
		 &(global_widgets[CONTROL_PANEL].widget)->style->
		 bg[GTK_STATE_NORMAL], ICON_GRP),
     TRUE, FALSE);
  child_node[3] = gtk_ctree_insert_node 
    (system_tree, inserted_node, NULL, no_text, NO_PADDING,
     new_pixmap ((global_widgets[CONTROL_PANEL].widget)->window, 
		 &(global_widgets[CONTROL_PANEL].widget)->style->
		 bg[GTK_STATE_NORMAL], ICON_TBL),
     new_bitmap ((global_widgets[CONTROL_PANEL].widget)->window, 
		 &(global_widgets[CONTROL_PANEL].widget)->style->
		 bg[GTK_STATE_NORMAL], ICON_TBL),
     new_pixmap ((global_widgets[CONTROL_PANEL].widget)->window, 
		 &(global_widgets[CONTROL_PANEL].widget)->style->
		 bg[GTK_STATE_NORMAL], ICON_TBL),
     new_bitmap ((global_widgets[CONTROL_PANEL].widget)->window, 
		 &(global_widgets[CONTROL_PANEL].widget)->style->
		 bg[GTK_STATE_NORMAL], ICON_TBL),
     TRUE, FALSE);
  for (i = 0; i < 2; i++)
    {
      gtk_ctree_node_set_pixmap
	(system_tree, child_node[i], 1,
	 new_pixmap ((global_widgets[CONTROL_PANEL].widget)->window, 
		     &(global_widgets[CONTROL_PANEL].widget)->style->
		     bg[GTK_STATE_NORMAL], ICON_CHECK_NOT_OK),
	 new_bitmap ((global_widgets[CONTROL_PANEL].widget)->window, 
		     &(global_widgets[CONTROL_PANEL].widget)->style->
		     bg[GTK_STATE_NORMAL], ICON_CHECK_NOT_OK));
    }

  /* Initialize and attach the data structures and their wrappers to the nodes */
  structure_object = g_new0 (Structure, 1);
  structure_phy = g_new0 (Structure, 1);
  structure_ki = g_new0 (Structure, 1);
  object = g_new0 (PhyObject, 1);
  object->object_type = PTOBJECT;
  object->active = TRUE;
  phy = g_new0 (PtObjectPhy, 1);
  ki = g_new0 (PtObjectKi, 1);
  structure_object->type = OBJECT;
  structure_object->structure = (gpointer) object;
  structure_phy->type = PHY;
  structure_phy->structure = (gpointer) phy;
  structure_ki->type = KI;
  structure_ki->structure = (gpointer) ki;

  gtk_ctree_node_set_row_data_full (system_tree, inserted_node, structure_object,
				    (GtkDestroyNotify) destroy_structure);
  gtk_ctree_node_set_row_data_full (system_tree, child_node[0], structure_phy,
				    (GtkDestroyNotify) destroy_structure);
  gtk_ctree_node_set_row_data_full (system_tree, child_node[1], structure_ki,
				    (GtkDestroyNotify) destroy_structure);

  /* Set the selectable rows, if a new object has been added,
     the active notebook page is def */
  gtk_ctree_node_set_selectable (system_tree, inserted_node, TRUE);
  gtk_ctree_node_set_selectable (system_tree, child_node[0], TRUE);
  gtk_ctree_node_set_selectable (system_tree, child_node[1], TRUE);
  gtk_ctree_node_set_selectable (system_tree, child_node[2], FALSE);
  gtk_ctree_node_set_selectable (system_tree, child_node[3], FALSE);
 
  /* Make the new object selected, this emits the tree_select_row signal
   which is not the one being handled, so emit the select_row signal as well */
  gtk_ctree_select (system_tree, inserted_node);
  gtk_signal_emit_by_name (GTK_OBJECT (system_tree), "select-row",
			   GTK_OBJECT (system_tree), 0, 0, NULL, NULL);
  gtk_clist_thaw (GTK_CLIST (system_tree));
}

/* Usefull for loading a system from an xml file. The copy element of the popup
   menu should probably use this rather than ... */
void pt_obj_append (PhyObject *object,
		    PtObjectPhy *phy,
		    PtObjectKi *ki)
{
  GtkCTree *system_tree = GTK_CTREE (global_widgets[SYSTEM_TREE].widget);
  GtkCTreeNode *inserted_node;
  GtkCTreeNode *child_node[4];
  gchar *no_text[] = {NO_TEXT, NULL, NULL, NULL, NULL};
  Structure *structure_object = NULL, *structure_phy = NULL, *structure_ki = NULL;

  gtk_clist_freeze (GTK_CLIST (system_tree));

  /* Append the nodes but don't set all the icons now */
  inserted_node = gtk_ctree_insert_node
    (system_tree, NULL, NULL, no_text, NO_PADDING, 
     new_pixmap ((global_widgets[CONTROL_PANEL].widget)->window, 
		 &(global_widgets[CONTROL_PANEL].widget)->style->
		 bg[GTK_STATE_NORMAL], ICON_PT_OBJ),
     new_bitmap ((global_widgets[CONTROL_PANEL].widget)->window, 
		 &(global_widgets[CONTROL_PANEL].widget)->style->
		 bg[GTK_STATE_NORMAL], ICON_PT_OBJ),
     new_pixmap ((global_widgets[CONTROL_PANEL].widget)->window, 
		 &(global_widgets[CONTROL_PANEL].widget)->style->
		 bg[GTK_STATE_NORMAL], ICON_PT_OBJ),
     new_bitmap ((global_widgets[CONTROL_PANEL].widget)->window, 
		 &(global_widgets[CONTROL_PANEL].widget)->style->
		 bg[GTK_STATE_NORMAL], ICON_PT_OBJ),
     FALSE, TRUE);
  child_node[0] =  gtk_ctree_insert_node 
    (system_tree, inserted_node, NULL, no_text, NO_PADDING,
     new_pixmap ((global_widgets[CONTROL_PANEL].widget)->window, 
		 &(global_widgets[CONTROL_PANEL].widget)->style->
		 bg[GTK_STATE_NORMAL], ICON_PHI),
     new_bitmap ((global_widgets[CONTROL_PANEL].widget)->window, 
		 &(global_widgets[CONTROL_PANEL].widget)->style->
		 bg[GTK_STATE_NORMAL], ICON_PHI),
     new_pixmap ((global_widgets[CONTROL_PANEL].widget)->window, 
		 &(global_widgets[CONTROL_PANEL].widget)->style->
		 bg[GTK_STATE_NORMAL], ICON_PHI),
     new_bitmap ((global_widgets[CONTROL_PANEL].widget)->window, 
		 &(global_widgets[CONTROL_PANEL].widget)->style->
		 bg[GTK_STATE_NORMAL], ICON_PHI),
     TRUE, FALSE);
  child_node[1] = gtk_ctree_insert_node 
    (system_tree, inserted_node, NULL, no_text, NO_PADDING,
     new_pixmap ((global_widgets[CONTROL_PANEL].widget)->window, 
		 &(global_widgets[CONTROL_PANEL].widget)->style->
		 bg[GTK_STATE_NORMAL], ICON_CHI),
     new_bitmap ((global_widgets[CONTROL_PANEL].widget)->window, 
		 &(global_widgets[CONTROL_PANEL].widget)->style->
		 bg[GTK_STATE_NORMAL], ICON_CHI),
     new_pixmap ((global_widgets[CONTROL_PANEL].widget)->window, 
		 &(global_widgets[CONTROL_PANEL].widget)->style->
		 bg[GTK_STATE_NORMAL], ICON_CHI),
     new_bitmap ((global_widgets[CONTROL_PANEL].widget)->window, 
		 &(global_widgets[CONTROL_PANEL].widget)->style->
		 bg[GTK_STATE_NORMAL], ICON_CHI),
     TRUE, FALSE);
  child_node[2] = gtk_ctree_insert_node 
    (system_tree, inserted_node, NULL, no_text, NO_PADDING,
     new_pixmap ((global_widgets[CONTROL_PANEL].widget)->window, 
		 &(global_widgets[CONTROL_PANEL].widget)->style->
		 bg[GTK_STATE_NORMAL], ICON_GRP),
     new_bitmap ((global_widgets[CONTROL_PANEL].widget)->window, 
		 &(global_widgets[CONTROL_PANEL].widget)->style->
		 bg[GTK_STATE_NORMAL], ICON_GRP),
     new_pixmap ((global_widgets[CONTROL_PANEL].widget)->window, 
		 &(global_widgets[CONTROL_PANEL].widget)->style->
		 bg[GTK_STATE_NORMAL], ICON_GRP),
     new_bitmap ((global_widgets[CONTROL_PANEL].widget)->window, 
		 &(global_widgets[CONTROL_PANEL].widget)->style->
		 bg[GTK_STATE_NORMAL], ICON_GRP),
     TRUE, FALSE);
  child_node[3] = gtk_ctree_insert_node 
    (system_tree, inserted_node, NULL, no_text, NO_PADDING,
     new_pixmap ((global_widgets[CONTROL_PANEL].widget)->window, 
		 &(global_widgets[CONTROL_PANEL].widget)->style->
		 bg[GTK_STATE_NORMAL], ICON_TBL),
     new_bitmap ((global_widgets[CONTROL_PANEL].widget)->window, 
		 &(global_widgets[CONTROL_PANEL].widget)->style->
		 bg[GTK_STATE_NORMAL], ICON_TBL),
     new_pixmap ((global_widgets[CONTROL_PANEL].widget)->window, 
		 &(global_widgets[CONTROL_PANEL].widget)->style->
		 bg[GTK_STATE_NORMAL], ICON_TBL),
     new_bitmap ((global_widgets[CONTROL_PANEL].widget)->window, 
		 &(global_widgets[CONTROL_PANEL].widget)->style->
		 bg[GTK_STATE_NORMAL], ICON_TBL),
     TRUE, FALSE);
  
  /* Initialize and attach the data structures and their wrappers to the nodes */
  structure_object = g_new0 (Structure, 1);
  structure_phy = g_new0 (Structure, 1);
  structure_ki = g_new0 (Structure, 1);
  structure_object->type = OBJECT;
  structure_object->structure = (gpointer) object;
  structure_phy->type = PHY;
  structure_phy->structure = (gpointer) phy;
  structure_ki->type = KI;
  structure_ki->structure = (gpointer) ki;

  gtk_ctree_node_set_row_data_full (system_tree, inserted_node, structure_object,
				    (GtkDestroyNotify) destroy_structure);
  gtk_ctree_node_set_row_data_full (system_tree, child_node[0], structure_phy,
				    (GtkDestroyNotify) destroy_structure);
  gtk_ctree_node_set_row_data_full (system_tree, child_node[1], structure_ki,
				    (GtkDestroyNotify) destroy_structure);

  /* Set the selectable rows, if a new object has been added,
     the active notebook page is def */
  gtk_ctree_node_set_selectable (system_tree, inserted_node, TRUE);
  gtk_ctree_node_set_selectable (system_tree, child_node[0], TRUE);
  gtk_ctree_node_set_selectable (system_tree, child_node[1], TRUE);
  gtk_ctree_node_set_selectable (system_tree, child_node[2], FALSE);
  gtk_ctree_node_set_selectable (system_tree, child_node[3], FALSE);
 
  /* Make the new object selected, this emits the tree_select_row signal
   which is not the one being handled, so emit the select_row signal as well */
  gtk_ctree_select (system_tree, inserted_node);
  gtk_signal_emit_by_name (GTK_OBJECT (system_tree), "select-row",
			   GTK_OBJECT (system_tree), 0, 0, NULL, NULL);

  /* Now set the icons, checking the active flag and the various ok flags */
  if (object->active == TRUE)
    gtk_ctree_node_set_pixmap
      (system_tree, inserted_node, 2,
       new_pixmap ((global_widgets[CONTROL_PANEL].widget)->window, 
		   &(global_widgets[CONTROL_PANEL].widget)->style->
		   bg[GTK_STATE_NORMAL], ICON_OK),
       new_bitmap ((global_widgets[CONTROL_PANEL].widget)->window, 
		   &(global_widgets[CONTROL_PANEL].widget)->style->
		   bg[GTK_STATE_NORMAL], ICON_OK));
  else
    gtk_ctree_node_set_pixmap
      (system_tree, inserted_node, 2,
       new_pixmap ((global_widgets[CONTROL_PANEL].widget)->window, 
		   &(global_widgets[CONTROL_PANEL].widget)->style->
		   bg[GTK_STATE_NORMAL], ICON_NOT_OK),
       new_bitmap ((global_widgets[CONTROL_PANEL].widget)->window, 
		   &(global_widgets[CONTROL_PANEL].widget)->style->
		   bg[GTK_STATE_NORMAL], ICON_NOT_OK));

  if (object->ok == TRUE)
    gtk_ctree_node_set_pixmap
      (system_tree, inserted_node, 1,
       new_pixmap ((global_widgets[CONTROL_PANEL].widget)->window, 
		   &(global_widgets[CONTROL_PANEL].widget)->style->
		   bg[GTK_STATE_NORMAL], ICON_CHECK_OK),
       new_bitmap ((global_widgets[CONTROL_PANEL].widget)->window, 
		   &(global_widgets[CONTROL_PANEL].widget)->style->
		   bg[GTK_STATE_NORMAL], ICON_CHECK_OK));
  else
    gtk_ctree_node_set_pixmap
      (system_tree, inserted_node, 1,
       new_pixmap ((global_widgets[CONTROL_PANEL].widget)->window, 
		   &(global_widgets[CONTROL_PANEL].widget)->style->
		   bg[GTK_STATE_NORMAL], ICON_CHECK_NOT_OK),
       new_bitmap ((global_widgets[CONTROL_PANEL].widget)->window, 
		   &(global_widgets[CONTROL_PANEL].widget)->style->
		   bg[GTK_STATE_NORMAL], ICON_CHECK_NOT_OK));
  
  if (phy->ok == TRUE)
    gtk_ctree_node_set_pixmap
      (system_tree, child_node[0], 1,
       new_pixmap ((global_widgets[CONTROL_PANEL].widget)->window, 
		   &(global_widgets[CONTROL_PANEL].widget)->style->
		   bg[GTK_STATE_NORMAL], ICON_CHECK_OK),
       new_bitmap ((global_widgets[CONTROL_PANEL].widget)->window, 
		   &(global_widgets[CONTROL_PANEL].widget)->style->
		   bg[GTK_STATE_NORMAL], ICON_CHECK_OK));
  else
    gtk_ctree_node_set_pixmap
      (system_tree, child_node[0], 1,
       new_pixmap ((global_widgets[CONTROL_PANEL].widget)->window, 
		   &(global_widgets[CONTROL_PANEL].widget)->style->
		   bg[GTK_STATE_NORMAL], ICON_CHECK_NOT_OK),
       new_bitmap ((global_widgets[CONTROL_PANEL].widget)->window, 
		   &(global_widgets[CONTROL_PANEL].widget)->style->
		   bg[GTK_STATE_NORMAL], ICON_CHECK_NOT_OK));

  if (ki->ok == TRUE)
    gtk_ctree_node_set_pixmap
      (system_tree, child_node[1], 1,
       new_pixmap ((global_widgets[CONTROL_PANEL].widget)->window, 
		   &(global_widgets[CONTROL_PANEL].widget)->style->
		   bg[GTK_STATE_NORMAL], ICON_CHECK_OK),
       new_bitmap ((global_widgets[CONTROL_PANEL].widget)->window, 
		   &(global_widgets[CONTROL_PANEL].widget)->style->
		   bg[GTK_STATE_NORMAL], ICON_CHECK_OK));
  else
    gtk_ctree_node_set_pixmap
      (system_tree, child_node[1], 1,
       new_pixmap ((global_widgets[CONTROL_PANEL].widget)->window, 
		   &(global_widgets[CONTROL_PANEL].widget)->style->
		   bg[GTK_STATE_NORMAL], ICON_CHECK_NOT_OK),
       new_bitmap ((global_widgets[CONTROL_PANEL].widget)->window, 
		   &(global_widgets[CONTROL_PANEL].widget)->style->
		   bg[GTK_STATE_NORMAL], ICON_CHECK_NOT_OK));

  /* Also insert the text */
  if (object->name)
    gtk_ctree_node_set_text (system_tree, inserted_node, 3, object->name);
  if (object->comment)
    gtk_ctree_node_set_text (system_tree, inserted_node, 4, object->comment);
  if (phy->comment)
    gtk_ctree_node_set_text (system_tree, child_node[0], 4, phy->comment);
  if (ki->comment)
    gtk_ctree_node_set_text (system_tree, child_node[1], 4, ki->comment);
  
  gtk_clist_thaw (GTK_CLIST (system_tree));
}

/* Use this function to change what's displayed in the control
   panel depending on which row is selected */
void display_selected_row (GtkObject *object,
			   gint row,
			   gint column,
			   GdkEventButton *event,
			   gpointer data)
{
  GtkCList *system_list = GTK_CLIST (object);
  Structure *structure = NULL;
  GtkWidget *label = NULL;
  GtkEntry *entry = NULL;
  GtkCTreeNode *parent = NULL, *node = NULL;
  GList *control_panel_children = NULL, *child = NULL;
  gchar *name = NULL, *value = NULL;
  gint i;

  if (!system_list->selection) return;
  if (!global_widgets[CONTROL_PANEL].exists) return;
   
  node = GTK_CTREE_NODE (system_list->selection->data);
  structure = (Structure*)
    gtk_ctree_node_get_row_data (GTK_CTREE (system_list), node);
  switch (structure->type)
    {
    case OBJECT:
      {
	/* Don't display anything in this case,
	   leave room for the system tree */
	control_panel_children = gtk_container_children
	  (GTK_CONTAINER (global_widgets[CONTROL_PANEL].widget));
	control_panel_children = gtk_container_children
	  (GTK_CONTAINER (control_panel_children->data));
	control_panel_children = control_panel_children->next;
	gtk_widget_hide (GTK_WIDGET (control_panel_children->data));
	gtk_widget_hide (GTK_WIDGET (control_panel_children->next->data));
      }
      break;
    case PHY:
      {
	PtObjectPhy *phy = NULL;
	
	/* Display the name and the physical entries of
	   the current object */
	phy = (PtObjectPhy*) structure->structure;
	parent = GTK_CTREE_ROW (node)->parent;
	gtk_ctree_node_get_text (GTK_CTREE (system_list),
				 parent, 3, &name); 
	control_panel_children = gtk_container_children
	  (GTK_CONTAINER (global_widgets[CONTROL_PANEL].widget));
	control_panel_children = gtk_container_children
	  (GTK_CONTAINER (control_panel_children->data));
	control_panel_children = control_panel_children->next;
	gtk_widget_show (GTK_WIDGET (control_panel_children->data));
	gtk_widget_hide (GTK_WIDGET (control_panel_children->next->data));
	label = GTK_WIDGET
	  ((gtk_container_children
	    (GTK_CONTAINER 
	     ((gtk_container_children 
	       (GTK_CONTAINER 
		(control_panel_children->data)))->data)))->data);
	gtk_label_set_text (GTK_LABEL (label), name);
	/* Insert the current values, if any, in the entries */
	control_panel_children = 
	  gtk_container_children (GTK_CONTAINER ((gtk_container_children
	  (GTK_CONTAINER (control_panel_children->data)))->data));
	control_panel_children = control_panel_children->next->next;
	/* Mass */
	entry = GTK_ENTRY
	  ((gtk_container_children 
	   (GTK_CONTAINER 
	    ((gtk_container_children 
	      (GTK_CONTAINER (control_panel_children->data)))->data)))->data);
	if (phy->mass_ok == TRUE)
	  {
	    value = g_strdup_printf ("%e", phy->mass);
	    gtk_entry_set_text (entry, value);
	  }
	else gtk_entry_set_text (entry, NO_TEXT);
	/* Dimension */
	entry = GTK_ENTRY
	  ((gtk_container_children 
	   (GTK_CONTAINER 
	    ((gtk_container_children 
	      (GTK_CONTAINER (control_panel_children->data)))->next->data)))->data);
	if (phy->dimension_exists == TRUE)
	  {
	    value = g_strdup_printf ("%e", phy->dimension);
	    gtk_entry_set_text (entry, value);
	  }
	else gtk_entry_set_text (entry, NO_TEXT);
      }
      break;
    case KI:
      {
	PtObjectKi *ki = NULL;
	
	ki = (PtObjectKi*) structure->structure;
	parent = GTK_CTREE_ROW (node)->parent;
	gtk_ctree_node_get_text (GTK_CTREE (system_list),
				 parent, 3, &name);
	control_panel_children = gtk_container_children
	  (GTK_CONTAINER (global_widgets[CONTROL_PANEL].widget));
	control_panel_children = gtk_container_children
	  (GTK_CONTAINER (control_panel_children->data));
	control_panel_children = control_panel_children->next;
	gtk_widget_hide (GTK_WIDGET (control_panel_children->data));
	control_panel_children = control_panel_children->next;
	gtk_widget_show (GTK_WIDGET (control_panel_children->data));
	child = 
	  gtk_container_children
	    (GTK_CONTAINER
	      ((gtk_container_children
		 (GTK_CONTAINER 
		  (control_panel_children->data)))->data));
	for (i = 0; i < 18; i++)
	  child = child->next;
	label = GTK_WIDGET (child->data);
	if (!name)
	  gtk_label_set_text (GTK_LABEL (label), NO_TEXT);
	gtk_label_set_text (GTK_LABEL (label), name);
      	/* Insert the current values, if any, in the entries */
	entry = GTK_ENTRY (child->prev->prev->prev->prev->data);
	if (ki->position_x_ok == TRUE)
	  {
	    value = g_strdup_printf ("%e", ki->positions[X]);
	    gtk_entry_set_text (entry, value);
	  }
	else gtk_entry_set_text (entry, NO_TEXT);
	entry = GTK_ENTRY (child->prev->prev->prev->prev->prev->prev->data);
	if (ki->position_y_ok == TRUE)
	  {
	    value = g_strdup_printf ("%e", ki->positions[Y]);
	    gtk_entry_set_text (entry, value);
	  }
	else gtk_entry_set_text (entry, NO_TEXT);
	entry = GTK_ENTRY (child->prev->prev->prev->prev
			   ->prev->prev->prev->prev->data);
	if (ki->position_z_ok == TRUE)
	  {
	    value = g_strdup_printf ("%e", ki->positions[Z]);
	    gtk_entry_set_text (entry, value);
	  }
	else gtk_entry_set_text (entry, NO_TEXT);
	entry = GTK_ENTRY (child->prev->prev->prev->prev->prev->prev
			   ->prev->prev->prev->prev->data);
	if (ki->speed_x_ok == TRUE)
	  {
	    value = g_strdup_printf ("%e", ki->speeds[X]);
	    gtk_entry_set_text (entry, value);
	  }
	else gtk_entry_set_text (entry, NO_TEXT);
	entry = GTK_ENTRY (child->prev->prev->prev->prev->prev->prev
			   ->prev->prev->prev->prev->prev->prev->data);
	if (ki->speed_y_ok == TRUE)
	  {
	    value = g_strdup_printf ("%e", ki->speeds[Y]);
	    gtk_entry_set_text (entry, value);
	  }
	else gtk_entry_set_text (entry, NO_TEXT);
	entry = GTK_ENTRY (child->prev->prev->prev->prev->prev->prev->prev
			   ->prev->prev->prev->prev->prev->prev->prev->data);
	if (ki->speed_z_ok == TRUE)
	  {
	    value = g_strdup_printf ("%e", ki->speeds[Z]);
	    gtk_entry_set_text (entry, value);
	  }
	else gtk_entry_set_text (entry, NO_TEXT);
      }
      break;
    }
}

/* Use this function to change what's displayed in the
   control panel depending on which notebook page is active */
void control_panel_display (GtkNotebook *notebook,
			    GtkNotebookPage *page,
			    gint page_num,
			    gpointer user_data)
{
  GtkCTree *system_tree = GTK_CTREE (global_widgets[SYSTEM_TREE].widget);
  GtkWidget *def_vbox = GTK_WIDGET (user_data);
  GList *control_panel_children = NULL;
  GtkCTreeNode *node = NULL, *parent = NULL;

  /* Update the control panel window's content depending on the
     active tree page */
  control_panel_children = gtk_container_children (GTK_CONTAINER (def_vbox));
  /* Skip the tree, which should always be displayed */
  control_panel_children = control_panel_children->next;
  /* Get the first node of the tree */
  parent = gtk_ctree_node_nth (system_tree, 0);

  switch (page_num)
    {
    case 0:
      /* Def page */
      gtk_widget_show (GTK_WIDGET (control_panel_children->data));
      control_panel_children = control_panel_children->next;
      control_panel_children = control_panel_children->next;
      control_panel_children = control_panel_children->next;
      while (control_panel_children != NULL)
	{
	  gtk_widget_hide (GTK_WIDGET (control_panel_children->data));
	  control_panel_children = control_panel_children->next; 
	}
      while (parent)
	{ 
	  node = parent;
	  gtk_ctree_node_set_selectable (system_tree, node, TRUE);
	  node = GTK_CTREE_ROW (node)->children;
	  gtk_ctree_node_set_selectable (system_tree, node, TRUE);
	  node = GTK_CTREE_NODE_NEXT (node);
	  gtk_ctree_node_set_selectable (system_tree, node, TRUE);
	  node = GTK_CTREE_NODE_NEXT (node);
	  gtk_ctree_node_set_selectable (system_tree, node, FALSE);
	  node = GTK_CTREE_NODE_NEXT (node);
	  gtk_ctree_node_set_selectable (system_tree, node, FALSE);
	  parent = GTK_CTREE_ROW (parent)->sibling;
	}      
      break;
    case 1:
      /* Grp page */
      gtk_widget_hide (GTK_WIDGET (control_panel_children->data));
      /* control_panel_children = control_panel_children->next;
	 gtk_widget_show (GTK_WIDGET (control_panel_children->data));
	 control_panel_children = control_panel_children->next;
	 while (control_panel_children != NULL)
	 {
	 gtk_widget_hide (GTK_WIDGET (control_panel_children->data));
	 control_panel_children = control_panel_children->next; 
	 } */
      break;
    case 2:
      /* Tbl page */
      gtk_widget_hide (GTK_WIDGET (control_panel_children->data));
      /* control_panel_children = control_panel_children->next;
	 gtk_widget_hide (GTK_WIDGET (control_panel_children->data));
	 control_panel_children = control_panel_children->next;
	 gtk_widget_show (GTK_WIDGET (control_panel_children->data));
	 control_panel_children = control_panel_children->next;
	 gtk_widget_hide (GTK_WIDGET (control_panel_children->data)); */
      break;
    case 3: 
      /* Scr page */
      gtk_widget_hide (GTK_WIDGET (control_panel_children->data));
      /* control_panel_children = control_panel_children->next;
	 gtk_widget_hide (GTK_WIDGET (control_panel_children->data));      
	 control_panel_children = control_panel_children->next;
	 gtk_widget_hide (GTK_WIDGET (control_panel_children->data));
	 control_panel_children = control_panel_children->next;
	 gtk_widget_show (GTK_WIDGET (control_panel_children->data));
	 control_panel_children = control_panel_children->next; */
      break; 
    }
  /* Do not free widgets here or else the next switch-page signal cannot access
     the proper data structure */
}  

void measure_mode (GtkWidget *widget,
		   gpointer *data)
{
  GtkWidget *menu = NULL;
  GtkItemFactory *menu_factory = NULL;
  GtkAccelGroup *menu_accel_group = gtk_accel_group_new ();
  GtkItemFactoryEntry menu_items[] =
  {
    { _MENU_MEASURE_MEAN,          NULL,   update_measure,          0,  NULL},
    { _MENU_MEASURE_MEDIAN,        NULL,   update_measure,          1,  NULL},
    { _MENU_MEASURE_MAX,           NULL,   update_measure,          2,  NULL},
    { _MENU_MEASURE_MIN,           NULL,   update_measure,          3,  NULL}
  };

  /* Build the menu */
  menu_factory = gtk_item_factory_new (GTK_TYPE_MENU, "<main>",
				       menu_accel_group);
  gtk_item_factory_create_items (menu_factory,
				 sizeof (menu_items) / sizeof (menu_items[0]),
				 menu_items, data);
  menu = gtk_item_factory_get_widget (menu_factory, "<main>");
  /* For standard callbacks, set them up manually 
     item = gtk_item_factory_get_widget (menu_factory, NEW_PT_OBJ);
     gtk_signal_connect (GTK_OBJECT (item), "activate", 
     GTK_SIGNAL_FUNC (pt_obj_new), system_list); */
  /* Display the menu */
  gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL, 
		  0, 0);
  
}

void update_measure (gpointer *data,
		     guint callback_action,
		     GtkWidget *widget)
{
  TwoWidgets *widgets;
  GList *button_label;
  GtkWidget *label;

  widgets = (TwoWidgets*) data;
  button_label = gtk_container_children (GTK_CONTAINER (widgets->widget1));
  gtk_container_remove (GTK_CONTAINER (widgets->widget1),
			GTK_WIDGET (button_label->data));

  switch (callback_action)
    {
    case 0:
      /* Change the label on the button */
      label = gtk_label_new (MENU_MEASURE_MEAN);
      gtk_container_add (GTK_CONTAINER (widgets->widget1), label);
      /* Change the measure displayed
	 Use widgets->widget2 and access the system_tree to make the
	 computation. */
      break;
    case 1:
      label = gtk_label_new (MENU_MEASURE_MEDIAN);
      gtk_container_add (GTK_CONTAINER (widgets->widget1), label);
      break;
    case 2:
      label = gtk_label_new (MENU_MEASURE_MAX);
      gtk_container_add (GTK_CONTAINER (widgets->widget1), label);
      break;
    case 3:
      label = gtk_label_new (MENU_MEASURE_MIN);
      gtk_container_add (GTK_CONTAINER (widgets->widget1), label);
      break;
    } 

  gtk_widget_show (label);
}

void inserted_text_handler (GtkEntry    *entry,
			  const gchar *text,
			  gint         length,
			  gint        *position,
			  gpointer     data)
{
  GtkEditable *editable = GTK_EDITABLE (entry);
  gint i, ascii;

  /* Check for allowed characters */
  for (i = 0; i < length; i++)
    {
      ascii = (gint) text[i];
      if ((ascii < 48 && ascii != 43 && ascii != 45 && ascii != 46)
	  || (ascii > 57 && ascii != 101))
	{
	  gtk_signal_handler_block_by_func
	    (GTK_OBJECT (editable),
	     GTK_SIGNAL_FUNC (inserted_text_handler),
	     data);
	  gtk_editable_insert_text (editable, text,
				    length - 1 , position);
	  gtk_signal_handler_unblock_by_func 
	    (GTK_OBJECT (editable),
	     GTK_SIGNAL_FUNC (inserted_text_handler),
	     data);
	  gtk_signal_emit_stop_by_name (GTK_OBJECT (editable),
					"insert_text");
	}
    }  
 }

void changed_text_handler (GtkEntry *entry,
			   gpointer data)
{
  GtkCTreeNode *selected = NULL;
  GtkCTree *system_tree = GTK_CTREE (global_widgets[SYSTEM_TREE].widget);
  Structure* structure = NULL;
  gint id = GPOINTER_TO_INT (data), i;
  gchar *text = gtk_entry_get_text (entry); 
  gdouble value;
  
  /* Checks that the entry is a number of the form (+-)123e(+-)34,
   if not returns the value corresponding to the valid part of the string */
  value = (gdouble) atof (text);
  /* Update the data structure in the tree */
  if (GTK_CLIST (system_tree)->selection)
    selected = GTK_CTREE_NODE (GTK_CLIST (system_tree)
			       ->selection->data);
  else return;

  switch (id)
    {
    case E_MASS:
      {
	PtObjectPhy *phy = NULL;
	
	if (text_insert_and_check (entry, value, TRUE, FALSE) == FALSE)
	  return;
	else
	  {
	    structure = (Structure*)
	      gtk_ctree_node_get_row_data (system_tree, selected);
	    phy = (PtObjectPhy*) structure->structure;
	    phy->mass = value;
	    phy->mass_ok = TRUE;
	  }
	if (phy->mass_ok)
	  {
	    PtObjectKi *ki = (PtObjectKi*)
	      ((Structure*) gtk_ctree_node_get_row_data 
	       (system_tree, GTK_CTREE_NODE_NEXT (selected)))->structure;
	    
	    phy->ok = TRUE;
	    gtk_ctree_node_set_pixmap
	      (GTK_CTREE (system_tree), selected, 1, 
	       new_pixmap ((global_widgets[CONTROL_PANEL].widget)->window, 
			   &(global_widgets[CONTROL_PANEL].widget)->style->
			   bg[GTK_STATE_NORMAL], ICON_CHECK_OK),
	       new_bitmap ((global_widgets[CONTROL_PANEL].widget)->window, 
			   &(global_widgets[CONTROL_PANEL].widget)->style->
			   bg[GTK_STATE_NORMAL], ICON_CHECK_OK));
	    if (ki->ok)
	      {
		GtkCTreeNode *parent = NULL;
		PhyObject *object = NULL;

		parent = GTK_CTREE_ROW (selected)->parent;
		gtk_ctree_node_set_pixmap
		  (GTK_CTREE (system_tree), parent, 1, 
		   new_pixmap ((global_widgets[CONTROL_PANEL].widget)->window, 
			       &(global_widgets[CONTROL_PANEL].widget)->style->
			       bg[GTK_STATE_NORMAL], ICON_CHECK_OK),
		   new_bitmap ((global_widgets[CONTROL_PANEL].widget)->window, 
			       &(global_widgets[CONTROL_PANEL].widget)->style->
			       bg[GTK_STATE_NORMAL], ICON_CHECK_OK));
		object = (PhyObject*) ((Structure*) gtk_ctree_node_get_row_data
				    (system_tree, parent))->structure;
		object->ok = TRUE;
	      }
	  }
      }
      break;
    case E_DIMENSION:
      {
	PtObjectPhy *phy = NULL;
	
	if (text_insert_and_check (entry, value, TRUE, FALSE) == FALSE)
	  return;
	else
	  {
	    structure = (Structure*)
	      gtk_ctree_node_get_row_data (system_tree, selected);
	    phy = (PtObjectPhy*) structure->structure;
	    phy->dimension = value;
	    phy->dimension_exists = TRUE;
	  }
      }
      break;
    case E_POSITION_X:
      {
	PtObjectKi *ki = NULL;
	
	if (text_insert_and_check (entry, value, FALSE, TRUE) == FALSE)
	  return;
	structure = (Structure*)
	  gtk_ctree_node_get_row_data (system_tree, selected);
	ki = (PtObjectKi*) structure->structure;
	ki->positions[X] = value;
	ki->position_x_ok = TRUE;
	
	if (ki->position_x_ok && ki->position_y_ok && ki->position_z_ok
	    && ki->speed_x_ok && ki->speed_y_ok && ki->speed_z_ok)
	  {
	    PtObjectPhy *phy = (PtObjectPhy*) 
	      ((Structure*) gtk_ctree_node_get_row_data
	       (system_tree,
		GTK_CTREE_NODE_PREV (selected)))->structure;
	    
	    ki->ok = TRUE;
	    gtk_ctree_node_set_pixmap
	      (GTK_CTREE (system_tree), selected, 1, 
	       new_pixmap ((global_widgets[CONTROL_PANEL].widget)->window, 
			   &(global_widgets[CONTROL_PANEL].widget)->style->
			   bg[GTK_STATE_NORMAL], ICON_CHECK_OK),
	       new_bitmap ((global_widgets[CONTROL_PANEL].widget)->window, 
			   &(global_widgets[CONTROL_PANEL].widget)->style->
			   bg[GTK_STATE_NORMAL], ICON_CHECK_OK));
	    if (phy->ok)
	      {
		GtkCTreeNode *parent = NULL;
		PhyObject *object = NULL;
		
		parent = GTK_CTREE_ROW (selected)->parent;
		gtk_ctree_node_set_pixmap
		(GTK_CTREE (system_tree), parent, 1, 
		 new_pixmap ((global_widgets[CONTROL_PANEL].widget)->window, 
			     &(global_widgets[CONTROL_PANEL].widget)->style->
			     bg[GTK_STATE_NORMAL], ICON_CHECK_OK),
		 new_bitmap ((global_widgets[CONTROL_PANEL].widget)->window, 
			     &(global_widgets[CONTROL_PANEL].widget)->style->
			     bg[GTK_STATE_NORMAL], ICON_CHECK_OK));
		object = (PhyObject*) ((Structure*) gtk_ctree_node_get_row_data
				    (system_tree, parent))->structure;
		object->ok = TRUE;
	      }
	  }
      }
      break;
    case E_POSITION_Y:
      {
	PtObjectKi *ki = NULL;
	
	if (text_insert_and_check (entry, value, FALSE, TRUE) == FALSE)
	  return;
	structure = (Structure*)
	  gtk_ctree_node_get_row_data (system_tree, selected);
	ki = (PtObjectKi*) structure->structure;
	ki->positions[Y] = value;
	ki->position_y_ok = TRUE;

	if (ki->position_x_ok && ki->position_y_ok && ki->position_z_ok
	    && ki->speed_x_ok && ki->speed_y_ok && ki->speed_z_ok)
	  {
	    PtObjectPhy *phy = (PtObjectPhy*) 
	      ((Structure*) gtk_ctree_node_get_row_data
	       (system_tree,
		GTK_CTREE_NODE_PREV (selected)))->structure;
	    
	    ki->ok = TRUE;
	    gtk_ctree_node_set_pixmap
	      (GTK_CTREE (system_tree), selected, 1, 
	       new_pixmap ((global_widgets[CONTROL_PANEL].widget)->window, 
			   &(global_widgets[CONTROL_PANEL].widget)->style->
			   bg[GTK_STATE_NORMAL], ICON_CHECK_OK),
	       new_bitmap ((global_widgets[CONTROL_PANEL].widget)->window, 
			   &(global_widgets[CONTROL_PANEL].widget)->style->
			   bg[GTK_STATE_NORMAL], ICON_CHECK_OK));
	    if (phy->ok)
	      {
		GtkCTreeNode *parent = NULL;
		PhyObject *object = NULL;
		
		parent = GTK_CTREE_ROW (selected)->parent;
		gtk_ctree_node_set_pixmap
		(GTK_CTREE (system_tree), parent, 1, 
		 new_pixmap ((global_widgets[CONTROL_PANEL].widget)->window, 
			     &(global_widgets[CONTROL_PANEL].widget)->style->
			     bg[GTK_STATE_NORMAL], ICON_CHECK_OK),
		 new_bitmap ((global_widgets[CONTROL_PANEL].widget)->window, 
			     &(global_widgets[CONTROL_PANEL].widget)->style->
			     bg[GTK_STATE_NORMAL], ICON_CHECK_OK));
		object = (PhyObject*) ((Structure*) gtk_ctree_node_get_row_data
				    (system_tree, parent))->structure;
		object->ok = TRUE;
	      }
	  }
      }
      break;
    case E_POSITION_Z:
      {
	PtObjectKi *ki = NULL;
	
	if (text_insert_and_check (entry, value, FALSE, TRUE) == FALSE)
	  return;
	structure = 
	  (Structure*) gtk_ctree_node_get_row_data (system_tree, selected);
	ki = (PtObjectKi*) structure->structure;
	ki->positions[Z] = value;
	ki->position_z_ok = TRUE;

	if (ki->position_x_ok && ki->position_y_ok && ki->position_z_ok
	    && ki->speed_x_ok && ki->speed_y_ok && ki->speed_z_ok)
	  {
	    PtObjectPhy *phy = (PtObjectPhy*) 
	      ((Structure*) gtk_ctree_node_get_row_data
	       (system_tree,
		GTK_CTREE_NODE_PREV (selected)))->structure;
	    
	    ki->ok = TRUE;
	    gtk_ctree_node_set_pixmap
	      (GTK_CTREE (system_tree), selected, 1, 
	       new_pixmap ((global_widgets[CONTROL_PANEL].widget)->window, 
			   &(global_widgets[CONTROL_PANEL].widget)->style->
			   bg[GTK_STATE_NORMAL], ICON_CHECK_OK),
	       new_bitmap ((global_widgets[CONTROL_PANEL].widget)->window, 
			   &(global_widgets[CONTROL_PANEL].widget)->style->
			   bg[GTK_STATE_NORMAL], ICON_CHECK_OK));
	    if (phy->ok)
	      {
		GtkCTreeNode *parent = NULL;
		PhyObject *object = NULL;
		
		parent = GTK_CTREE_ROW (selected)->parent;
		gtk_ctree_node_set_pixmap
		  (GTK_CTREE (system_tree), parent, 1, 
		   new_pixmap ((global_widgets[CONTROL_PANEL].widget)->window, 
			       &(global_widgets[CONTROL_PANEL].widget)->style->
			       bg[GTK_STATE_NORMAL], ICON_CHECK_OK),
		   new_bitmap ((global_widgets[CONTROL_PANEL].widget)->window, 
			       &(global_widgets[CONTROL_PANEL].widget)->style->
			       bg[GTK_STATE_NORMAL], ICON_CHECK_OK));
		object = (PhyObject*) ((Structure*) gtk_ctree_node_get_row_data
				    (system_tree, parent))->structure;
		object->ok = TRUE;
	      }
	  }
      }
      break;
    case E_SPEED_X:
      {
	PtObjectKi *ki = NULL;
	
	if (text_insert_and_check (entry, value, FALSE, TRUE) == FALSE)
	  return;
	structure = (Structure*) 
	  gtk_ctree_node_get_row_data (system_tree, selected);
	ki = (PtObjectKi*) structure->structure;
	ki->speeds[X] = value;
	ki->speed_x_ok = TRUE;

	if (ki->position_x_ok && ki->position_y_ok && ki->position_z_ok
	    && ki->speed_x_ok && ki->speed_y_ok && ki->speed_z_ok)
	  {
	    PtObjectPhy *phy = (PtObjectPhy*) 
	      ((Structure*) gtk_ctree_node_get_row_data
	       (system_tree,
		GTK_CTREE_NODE_PREV (selected)))->structure;
	    
	    ki->ok = TRUE;
	    gtk_ctree_node_set_pixmap
	      (GTK_CTREE (system_tree), selected, 1, 
	       new_pixmap ((global_widgets[CONTROL_PANEL].widget)->window, 
			   &(global_widgets[CONTROL_PANEL].widget)->style->
			   bg[GTK_STATE_NORMAL], ICON_CHECK_OK),
	       new_bitmap ((global_widgets[CONTROL_PANEL].widget)->window, 
			   &(global_widgets[CONTROL_PANEL].widget)->style->
			   bg[GTK_STATE_NORMAL], ICON_CHECK_OK));
	    if (phy->ok)
	      {
		GtkCTreeNode *parent = NULL;
		PhyObject *object = NULL;

		parent = GTK_CTREE_ROW (selected)->parent;
		gtk_ctree_node_set_pixmap
		  (GTK_CTREE (system_tree), parent, 1, 
		   new_pixmap ((global_widgets[CONTROL_PANEL].widget)->window, 
			       &(global_widgets[CONTROL_PANEL].widget)->style->
			       bg[GTK_STATE_NORMAL], ICON_CHECK_OK),
		   new_bitmap ((global_widgets[CONTROL_PANEL].widget)->window, 
			       &(global_widgets[CONTROL_PANEL].widget)->style->
			       bg[GTK_STATE_NORMAL], ICON_CHECK_OK));
		object = (PhyObject*) ((Structure*) gtk_ctree_node_get_row_data
				    (system_tree, parent))->structure;
		object->ok = TRUE;
	      }
	  }
      }
      break;
    case E_SPEED_Y:
      {
	PtObjectKi *ki = NULL;
	
	if (text_insert_and_check (entry, value, FALSE, TRUE) == FALSE)
	  return;
	structure = (Structure*) 
	  gtk_ctree_node_get_row_data (system_tree, selected);
	ki = (PtObjectKi*) structure->structure;
	ki->speeds[Y] = value;
	ki->speed_y_ok = TRUE;

	if (ki->position_x_ok && ki->position_y_ok && ki->position_z_ok
	    && ki->speed_x_ok && ki->speed_y_ok && ki->speed_z_ok)
	  {
	    PtObjectPhy *phy = (PtObjectPhy*) 
	      ((Structure*) gtk_ctree_node_get_row_data
	       (system_tree,
		GTK_CTREE_NODE_PREV (selected)))->structure;
	  
	    ki->ok = TRUE;
	    gtk_ctree_node_set_pixmap
	      (GTK_CTREE (system_tree), selected, 1, 
	       new_pixmap ((global_widgets[CONTROL_PANEL].widget)->window, 
			   &(global_widgets[CONTROL_PANEL].widget)->style->
			   bg[GTK_STATE_NORMAL], ICON_CHECK_OK),
	       new_bitmap ((global_widgets[CONTROL_PANEL].widget)->window, 
			   &(global_widgets[CONTROL_PANEL].widget)->style->
			   bg[GTK_STATE_NORMAL], ICON_CHECK_OK));
	    if (phy->ok)
	      {
		GtkCTreeNode *parent = NULL;
		PhyObject *object = NULL;
		
		parent = GTK_CTREE_ROW (selected)->parent;
		gtk_ctree_node_set_pixmap
		  (GTK_CTREE (system_tree), parent, 1, 
		   new_pixmap ((global_widgets[CONTROL_PANEL].widget)->window, 
			       &(global_widgets[CONTROL_PANEL].widget)->style->
			       bg[GTK_STATE_NORMAL], ICON_CHECK_OK),
		   new_bitmap ((global_widgets[CONTROL_PANEL].widget)->window, 
			       &(global_widgets[CONTROL_PANEL].widget)->style->
			       bg[GTK_STATE_NORMAL], ICON_CHECK_OK));
		object = (PhyObject*) ((Structure*) gtk_ctree_node_get_row_data
				    (system_tree, parent))->structure;
		object->ok = TRUE;
	      }
	  }
      }
      break;
    case E_SPEED_Z: 
      {
	PtObjectKi *ki = NULL;
	
	if (text_insert_and_check (entry, value, FALSE, TRUE) == FALSE)
	  return;
	structure = (Structure*) gtk_ctree_node_get_row_data (system_tree,
							      selected);
	ki = (PtObjectKi*) structure->structure;
	ki->speeds[Z] = value;
	ki->speed_z_ok = TRUE;

	if (ki->position_x_ok && ki->position_y_ok && ki->position_z_ok
	    && ki->speed_x_ok && ki->speed_y_ok && ki->speed_z_ok)
	 {
	   PtObjectPhy *phy = (PtObjectPhy*) 
	     ((Structure*) gtk_ctree_node_get_row_data
	      (system_tree,
	       GTK_CTREE_NODE_PREV (selected)))->structure;
	   
	   ki->ok = TRUE;
	   gtk_ctree_node_set_pixmap
	     (GTK_CTREE (system_tree), selected, 1, 
	      new_pixmap ((global_widgets[CONTROL_PANEL].widget)->window, 
			  &(global_widgets[CONTROL_PANEL].widget)->style->
			  bg[GTK_STATE_NORMAL], ICON_CHECK_OK),
	      new_bitmap ((global_widgets[CONTROL_PANEL].widget)->window, 
			  &(global_widgets[CONTROL_PANEL].widget)->style->
			  bg[GTK_STATE_NORMAL], ICON_CHECK_OK));
	   if (phy->ok)
	     {
	       GtkCTreeNode *parent = NULL;
	       PhyObject *object = NULL;
	       
	       parent = GTK_CTREE_ROW (selected)->parent;
	       gtk_ctree_node_set_pixmap
		 (GTK_CTREE (system_tree), parent, 1, 
		  new_pixmap ((global_widgets[CONTROL_PANEL].widget)->window, 
			      &(global_widgets[CONTROL_PANEL].widget)->style->
			      bg[GTK_STATE_NORMAL], ICON_CHECK_OK),
		  new_bitmap ((global_widgets[CONTROL_PANEL].widget)->window, 
			      &(global_widgets[CONTROL_PANEL].widget)->style->
			      bg[GTK_STATE_NORMAL], ICON_CHECK_OK));
		object = (PhyObject*) ((Structure*) gtk_ctree_node_get_row_data
				    (system_tree, parent))->structure;
		object->ok = TRUE;
	     }
	 }
      }
      break;
    case E_START:
      if (simu_parameters->stop_ok)
	{
	  if (simu_parameters->stop <= value)
	    {
	      GtkWidget *message = gtk_label_new (START_MESSAGE);
	      GtkWidget *ok_button = gtk_button_new_with_label (OK);
	      gint id = ERROR_DIALOG;
	      
	      if (global_widgets[id].exists)
		gtk_widget_destroy (global_widgets[id].widget);
	      SETUP_DIALOG (global_widgets[id].widget, ERROR,
			    GINT_TO_POINTER (id));
	      gtk_entry_set_text (entry, NO_TEXT);
	      
	      gtk_box_pack_start (GTK_BOX 
				  (GTK_DIALOG (global_widgets[id].widget)-> vbox),
				  message, FALSE, FALSE, NO_PADDING);
	      gtk_signal_connect (GTK_OBJECT (ok_button), "clicked",
				  GTK_SIGNAL_FUNC (window_destroy_signal), 
				  GINT_TO_POINTER (id));      
	      gtk_box_pack_start (GTK_BOX (GTK_DIALOG (global_widgets[id].widget
						       )-> action_area),
				  ok_button, FALSE, FALSE, NO_PADDING);
	      GTK_WIDGET_SET_FLAGS (ok_button, GTK_CAN_DEFAULT);
	      gtk_widget_grab_default (ok_button);
	      gtk_widget_show_all (global_widgets[id].widget);
	      break;
	    }
	}
      if (text_insert_and_check (entry, value, FALSE, TRUE) == FALSE)
	return;
      simu_parameters->start = value;
      simu_parameters->start_ok = TRUE;
      
      if (simu_parameters->start_ok && simu_parameters->stop_ok)
	simu_parameters->ok = TRUE;
      break;
    case E_STOP:
      if (simu_parameters->start_ok)
	{
	  if (simu_parameters->start >= value)
	    {
	      GtkWidget *message = gtk_label_new (STOP_MESSAGE);
	      GtkWidget *ok_button = gtk_button_new_with_label (OK);
	      gint id = ERROR_DIALOG;
	      
	      if (global_widgets[id].exists)
		gtk_widget_destroy (global_widgets[id].widget);
	      SETUP_DIALOG (global_widgets[id].widget, ERROR,
			    GINT_TO_POINTER (id));
	      gtk_entry_set_text (entry, NO_TEXT);
	      
	      gtk_box_pack_start 
		(GTK_BOX (GTK_DIALOG (global_widgets[id].widget)-> vbox),
		 message, FALSE, FALSE, NO_PADDING);
	      gtk_signal_connect (GTK_OBJECT (ok_button), "clicked",
				  GTK_SIGNAL_FUNC (window_destroy_signal), 
				  GINT_TO_POINTER (id));      
	      gtk_box_pack_start (GTK_BOX (GTK_DIALOG (global_widgets[id].widget
						       )-> action_area),
				  ok_button, FALSE, FALSE, NO_PADDING);
	      GTK_WIDGET_SET_FLAGS (ok_button, GTK_CAN_DEFAULT);
	      gtk_widget_grab_default (ok_button);
	      gtk_widget_show_all (global_widgets[id].widget);
	      break;
	    }
	}
      simu_parameters->stop = value;
      simu_parameters->stop_ok = TRUE;
      if (text_insert_and_check (entry, value, FALSE, TRUE) == FALSE)
	return;

      if (simu_parameters->start_ok && simu_parameters->stop_ok)
	simu_parameters->ok = TRUE;
      break;
    case E_N_STEP:
      /* Must be an integer */
      if (text_insert_and_check (entry, value, TRUE, FALSE) == FALSE)
	return;
      simu_parameters->n_step = (gint)value;
      simu_parameters->n_step_exists = TRUE;
      break;
    case E_RACCURACY:
      if (text_insert_and_check (entry, value, TRUE, FALSE) == FALSE)
	return;
      simu_parameters->raccuracy = value;
      simu_parameters->raccuracy_exists = TRUE;
      break;
    case E_AACCURACY:
      if (text_insert_and_check (entry, value, TRUE, FALSE) == FALSE)
	return;
      simu_parameters->aaccuracy = value;
      simu_parameters->aaccuracy_exists = TRUE;
      break;
    }

  /* Update the DEF_GL areas */
  if (id == E_POSITION_X || id == E_POSITION_Y || id == E_POSITION_Z ||
      id == E_SPEED_X    || id == E_SPEED_Y    || id == E_SPEED_Z)
    {
      def_mesh_create ();
      if (global_widgets[DEF_GL_X].exists)
	for (i = 0; i < 4; i++)
	  redraw_viewport (global_widgets[DEF_GL_X + i].widget);
    }
}

/* Should be modified to handle floats, integers... any various formats
   (specified as argument ?) */
gboolean text_insert_and_check (GtkEntry *entry, gdouble value, gboolean positive,
				gboolean zero)
{
  gchar *text;
  gchar *plus_infinity = "inf";
  gchar *minus_infinity = "-inf";

  if (positive && value < 0.0) 
    {
      SETUP_ERROR_DIALOG (ERROR, POSITIVE_VALUE_MESSAGE);
      return (FALSE);
    }
  if (!zero && value == 0.0)
    {
      SETUP_ERROR_DIALOG (ERROR, NON_ZERO_VALUE_MESSAGE);
      return (FALSE);
    }
  text = g_strdup_printf ("%e", value);
  if (g_strcasecmp (text, plus_infinity) == 0 ||
      g_strcasecmp (text, minus_infinity) == 0)
    {
      SETUP_ERROR_DIALOG (ERROR, INFINITE_VALUE_MESSAGE);
      return (FALSE);
    }
  gtk_entry_set_text (entry, text);
  return (TRUE);
}

/* Is indeed usefull for hiding data, but the corresponding redisplay 
   function has not yet been implemented */
void tbl_hide_column  (GtkCList *list, gint index, gpointer data)
{
  //  gtk_clist_set_column_visibility (list, index, FALSE);
  return;
}

/* Should be modified to allow for client-server communication and to make sure
   that the simulation goes all the way to the end of the interval specified
   by the user */
void simulate_newtonian (GtkButton *simulate_button,
			 gpointer data)
{
  GtkCTree *system_tree = GTK_CTREE (global_widgets[SYSTEM_TREE].widget);
  GtkCTreeNode *node = NULL;
  PhyObject *object;
  gint n_body = 0;
  gint i, k;
  gint n_step;
  gchar *results;

  node = gtk_ctree_node_nth (system_tree, 0);
  if (!node) return;

  /* Check the system tree */
  while (node)
    {
      object = (PhyObject*)
	((Structure*)
	 (gtk_ctree_node_get_row_data (system_tree, node)))->structure;

      /* Check to see if all active objects are ok */
      if (object->active)
	{
	  if (object->ok)
	    n_body++;
	  else
	    {
	      SETUP_ERROR_DIALOG (ERROR, OBJECT_NOT_OK_MESSAGE);
	      gtk_ctree_select (system_tree, node);
	      /* Make the selected node visible here */
	      return;
	    }
	}
      node = GTK_CTREE_NODE (GTK_CTREE_ROW (node)->sibling);
    }

  destroy_computation_structures ();

  /* Allocate input and output data structures and arrays of appropriate length */
  input = g_new0 (Input, 1);
  input->controls = (gpointer) g_new0 (NControl, 1);
  input->phy = (gpointer) g_new0 (NPhyInput, 1);
  input->ki = (gpointer) g_new0 (NKiInput, 1);
  ((NPhyInput*)input->phy)->masses = g_new0 (double, n_body);
  ((NPhyInput*)input->phy)->dimensions = g_new0 (double, n_body);
  /* Allocate these in such a way as to have contiguous arrays for positions
     and speeds */
  ((NKiInput*)input->ki)->positions = g_new0 (double, 6*n_body);
  ((NKiInput*)input->ki)->speeds = &(((NKiInput*)input->ki)->positions[3*n_body]);

  output = g_new0 (Output, 1);
  /* Nothing sophisticated needs to be done with the controls in the newtonian
     case */
  output->controls = (gpointer) input->controls;
  output->objects = (gpointer) g_new0 (NObjectOutput, n_body);
  if (!(n_step = simu_parameters->n_step)) n_step = DEFAULT_N_STEP;
  for (i = 0; i < n_body; i++)
    {
      ((NObjectOutput*)output->objects)[i].x = g_new0 (double, n_step);
      ((NObjectOutput*)output->objects)[i].y = g_new0 (double, n_step);
      ((NObjectOutput*)output->objects)[i].z = g_new0 (double, n_step);
      ((NObjectOutput*)output->objects)[i].vx = g_new0 (double, n_step);
      ((NObjectOutput*)output->objects)[i].vy = g_new0 (double, n_step);
      ((NObjectOutput*)output->objects)[i].vz = g_new0 (double, n_step);
    }

  /* Set up the simulation controls */
  input->n_body = n_body;
  output->n_body = n_body;
  
  if (simu_parameters->start_ok) 
      ((NControl*) input->controls)->start = simu_parameters->start;
  else return;
  if (simu_parameters->stop_ok)
      ((NControl*) input->controls)->stop = simu_parameters->stop;
  else return;
  if (simu_parameters->n_step_exists)
      ((NControl*) input->controls)->n_step = simu_parameters->n_step;
  else 
      ((NControl*) input->controls)->n_step = DEFAULT_N_STEP;
  ((NControl*) input->controls)->step_size =
    (simu_parameters->stop - simu_parameters->start) /
    (((NControl*) input->controls)->n_step);
  /* Should be:     (((NControl*) input->controls)->n_step - 1); in order to
     reach the end of the simulation interval but it messes up the grp display */
  if (simu_parameters->raccuracy_exists) 
      ((NControl*)input->controls)->raccuracy = simu_parameters->raccuracy;
  else ((NControl*)input->controls)->raccuracy = DEFAULT_RACCURACY;
  if (simu_parameters->aaccuracy_exists) 
    ((NControl*)input->controls)->aaccuracy = simu_parameters->aaccuracy;
  else ((NControl*)input->controls)->aaccuracy = DEFAULT_AACCURACY;

  /* Retrieve the phy and ki data */
  node = gtk_ctree_node_nth (system_tree, 0);
  i = 0;
  while (node)
    {
      object = (PhyObject*)
	((Structure*) 
	 (gtk_ctree_node_get_row_data (system_tree, node)))->structure;
      if (object->active)
	{
	  GtkCTreeNode *child = GTK_CTREE_ROW (node)->children;
	  PtObjectPhy *input_phy = (PtObjectPhy*)
	    ((Structure*) 
	     (gtk_ctree_node_get_row_data (system_tree, child)))->structure;
	  PtObjectKi *input_ki = NULL;

	  /* Retrieving the data from the object structure */
	  ((NObjectOutput*) output->objects)[i].name = object->name;
	  /* Retrieving the data from the phy structure */
	  ((NPhyInput*) input->phy)->masses[i] = input_phy->mass;
	  ((NObjectOutput*) output->objects)[i].mass = input_phy->mass;
	  if (input_phy->dimension_exists)
	   {
	     ((NPhyInput*) input->phy)->dimensions[i] = input_phy->dimension;
	     ((NObjectOutput*) output->objects)[i].dimension = 
	       input_phy->dimension;
	   }
	  else 
	    {
	      ((NObjectOutput*) output->objects)[i].dimension = 
		DEFAULT_OBJ_DIMENSION;
	      ((NPhyInput*) input->phy)->dimensions[i] = DEFAULT_OBJ_DIMENSION;
	    }
	  
	  /* Retrieving the data from the ki structure */
	  child = GTK_CTREE_NODE_NEXT (child);
	  input_ki = (PtObjectKi*) 
	    ((Structure*) 
	     (gtk_ctree_node_get_row_data (system_tree, child)))->structure;
	  for (k = 0; k < 3; k++)
	    {
	      ((NKiInput*) input->ki)->positions[3*i + k] = input_ki->positions[k];
	      ((NKiInput*) input->ki)->speeds[3*i + k] = input_ki->speeds[k];
	    }

	  /* Set the initial conditions in the output structure */
	  ((NObjectOutput*) output->objects)[i].x[0] = input_ki->positions[0];
	  ((NObjectOutput*) output->objects)[i].vx[0] = input_ki->speeds[0];
	  ((NObjectOutput*) output->objects)[i].y[0] = input_ki->positions[1];
	  ((NObjectOutput*) output->objects)[i].vy[0] = input_ki->speeds[1];
	  ((NObjectOutput*) output->objects)[i].z[0] = input_ki->positions[2];
	  ((NObjectOutput*) output->objects)[i].vz[0] = input_ki->speeds[2];
	  
	  i++;
	}
      node = GTK_CTREE_NODE (GTK_CTREE_ROW (node)->sibling);
    }

  /* Call the integrator. This should be done in a client-server way.
     Still update the GUI inteface if events are pending, use:
     if (gtk_events_pending ()) gtk_main_iteration (); to go through
     the gtk_main loop once. */

  i = dop853
    (6*input->n_body,  /* dimension of the system */
     newtonian_force,  /* function computing the value of f(x,y) */
     ((NControl*) input->controls)->start,             /* initial x-value */
     ((NKiInput*) input->ki)->positions,               /* initial values for y */
     ((NControl*) input->controls)->stop,              /* final x-value */
     &(((NControl*) input->controls)->raccuracy),      /* rel error tolerance */
     &(((NControl*) input->controls)->aaccuracy),      /* abs error tolerance */
     0,                /* scalar error tolerance */
     solout,           /* provides the numerical solution during integration */
     2,                /* switch for calling solout: dense output used */
     stdout,           /* messages stream */
     0,                /* rounding unit (use default setting) */
     0,                /* safety factor (use default setting) */
     0,                /* parameters for step size selection (use default setting) */
     0,                /* (use default settings) */
     0,                /* for stabilized step size control (use default setting) */
     0,                /* maximal step size (use default setting) */
     0,                /* initial step size (use default setting) */
     DEFAULT_MAX_N_STEP,       /* maximum number of allowed steps */
     0,                /* switch for choosing the coefficients (use default setting)*/
     0,                /* test for stiffness (use default setting) */
     6*input->n_body,  /* number of components for which dense outpout is required */
     NULL,             /* indexes of components for which dense output is required */
     6*input->n_body   /* declared length of icon */
     );    

  /* Stop if the computation was not successful */
  if (i != 1) 
    {
      SETUP_ERROR_DIALOG (ERROR, COMPUTATION_ERROR_MESSAGE);
      return;
    }
 
  /* Make the results available in the grp and the tbl sections */
  /* The grp section */
  grp_fill ();
   
  /* If the grp page is active, update the GtkGLAreas now 
     if (gtk_notebook_get_current_page
     (GTK_NOTEBOOK (global_widgets[NOTEBOOK].widget)) == 1) */
  
  /* The tbl section */
  tbl_fill ();

  /* Pop up a dialog box giving details on the computation */
  results = g_strdup_printf (COMPUTATION_STATISTICS_MESSAGE,
			     nfcnRead (),
			     nstepRead (),
			     naccptRead (),
			     nrejctRead ());
  SETUP_MESSAGE_DIALOG (WINDOW_STATISTICS, results);

}

/* Set up the OpenGL mesh for display in the grp viewports */
void grp_fill (void)
{
  int i;
  /* Realize the GRP_GL* widgets in case it has not yet been done */
  if (!GTK_WIDGET_REALIZED (global_widgets[GRP_GL_X].widget))
    {
      for (i = 0; i < 4; i++)
	gtk_widget_realize (global_widgets[GRP_GL_X + i].widget);
    } 

  /* Build the new display list */
  grp_mesh_create ();  
}

/* Put the numerical output data in the tbl page of the notebook */
void tbl_fill (void)
{
  GtkCList *tbl = GTK_CLIST (global_widgets[TBL].widget);
  NObjectOutput *objects = (NObjectOutput*) output->objects;
  gint i, j, k;
  gdouble time, energy, kinetic, gravitational, dist;
  gdouble *force, *position;
  gchar *data[N_TABLE_COLUMNS];
  gchar *global = "Global";


  gtk_clist_freeze (tbl);

  /* Remove what was already in the tbl page, if anything */
  /* This does not free all the memory allocated for the 
     content of the clist, testgtk does not free that memory 
     either, so how can I do it ? */
  tbl_new (NULL, NULL);

  /* Insert the new object data */
  for (i = 0; i < output->n_body; i++)
    {
      for (j = 0; j < ((NControl*) output->controls)->n_step; j++)
	{
	  /* Set all data fields to NO_TEXT */
	  for (k = 0; k < N_TABLE_COLUMNS; k++)
	    data[k] = NO_TEXT;

	  if (j == 0)
	    {
	      if (objects[i].name != NULL)
		data[C_OBJ] = g_strdup_printf ("%s", objects[i].name);
	      else
		data[C_OBJ] = g_strdup_printf ("Object %d", i + 1);
	    }
	  else data[C_OBJ] = NULL;
	  
	  /* Time, position and speed */
	  time = (gdouble) j*((NControl*) output->controls)->step_size;
	  data[C_T] = g_strdup_printf ("%g", time);
	  data[C_X] = g_strdup_printf ("%g", objects[i].x[j]);
	  data[C_Y] = g_strdup_printf ("%g", objects[i].y[j]);
	  data[C_Z] = g_strdup_printf ("%g", objects[i].z[j]);
	  data[C_VX] = g_strdup_printf ("%g", objects[i].vx[j]);
	  data[C_VY] = g_strdup_printf ("%g", objects[i].vy[j]);
	  data[C_VZ] = g_strdup_printf ("%g", objects[i].vz[j]);

	  /* Kinetic energy */
	  kinetic = (objects[i].mass *
		     (objects[i].vx[j]*objects[i].vx[j] +
		      objects[i].vy[j]*objects[i].vy[j] +
		      objects[i].vz[j]*objects[i].vz[j]));
	  kinetic /= 2.0;
	  data[C_K_ENERGY] = g_strdup_printf ("%g", kinetic);

	  /* Gravitational energy
	     It is not really the gravitational energy. This is probably not
	     really meaningful (ie suppress the corresponding column). */
	  gravitational = 0.0;
	  for (k = 0; k < output->n_body; k++)
	    {
	      if (k == i) continue;
	      dist = sqrt ((objects[i].x[j] - objects[k].x[j]) *
			   (objects[i].x[j] - objects[k].x[j]) +
			   (objects[i].y[j] - objects[k].y[j]) *
			   (objects[i].y[j] - objects[k].y[j]) +
			   (objects[i].z[j] - objects[k].z[j]) *
			   (objects[i].z[j] - objects[k].z[j]));
	      
	      gravitational -= (objects[i].mass * objects[k].mass)/dist;
	    }
	  
	  gravitational *= G_CONSTANT;
	  data[C_G_ENERGY] = g_strdup_printf ("%g", gravitational);
      
	  /* Global energy, not constant because bodies ARE exchanging energy (?)
	     In fact there is an additional term in the 'constant' energy 
	     formula for each body (corresponding to exchanges ?) */
	  energy = kinetic + gravitational;
	  data[C_ENERGY] = g_strdup_printf ("%g", energy);

	  gtk_clist_append (tbl, data);
	}
    }

  /* Insert the new global data */
  for (j = 0; j < ((NControl*) output->controls)->n_step; j++)
    {
      /* Set all data fields to NULL */
      memset (data, 0, N_TABLE_COLUMNS*sizeof (char*));

      if (j == 0)
	data[C_OBJ] = g_strdup_printf ("%s", global);
      else
	data[C_OBJ] = NULL;
	  
      /* The global object if the center of gravity, also add global 
	   energy, momentum...*/
      time = (gdouble) j*((NControl*) output->controls)->step_size;
      data[C_T] = g_strdup_printf ("%g", time);
      
      /* The system's kinetic energy */
      kinetic = 0.0;
      for (i = 0; i < output->n_body; i++)
	kinetic += (objects[i].mass *
		    (objects[i].vx[j]*objects[i].vx[j] +
		     objects[i].vy[j]*objects[i].vy[j] +
		     objects[i].vz[j]*objects[i].vz[j]));
      kinetic /= 2.0;
      data[C_K_ENERGY] = g_strdup_printf ("%g", kinetic);

      /* The system's gravitational energy */
      gravitational = 0.0;
      for (i = 1; i < output->n_body; i++)
	{
	  for (k = 0; k <= i - 1; k++)
	    {
	      dist = sqrt ((objects[i].x[j] - objects[k].x[j]) *
			   (objects[i].x[j] - objects[k].x[j]) +
			   (objects[i].y[j] - objects[k].y[j]) *
			   (objects[i].y[j] - objects[k].y[j]) +
			   (objects[i].z[j] - objects[k].z[j]) *
			   (objects[i].z[j] - objects[k].z[j]));
	      gravitational += (objects[i].mass * objects[k].mass)/dist;
	    }
	}
      gravitational *= (-1)*G_CONSTANT;
      data[C_G_ENERGY] = g_strdup_printf ("%g", gravitational);

      /* Global energy, MUST be constant */
      energy = kinetic + gravitational;
      data[C_ENERGY] = g_strdup_printf ("%g", energy);

      gtk_clist_append (tbl, data);
    }
  
  /* Force
     force = g_new0 (gdouble, 6*output->n_body);
     position = g_new0 (gdouble, 6*output->n_body);
     for (j = 0; j < ((NControl*) output->controls)->n_step; j++)
     {
     Set all data fields to NULL
     memset (data, 0, N_TABLE_COLUMNS*sizeof (char*));

     for (i = 0; i <output->n_body; i++)
     {
     position[3*i] = objects[i].x[j];
     position[3*i + 1] = objects[i].y[j];
     position[3*i + 2] = objects[i].z[j];
     }
   
     newtonian_force (6*output->n_body, 0, position, force);
      
     Now insert in the TBL
     for (i = 0; i < output->n_body; i++)
     {
     data[C_FORCE_X] = g_strdup_printf ("%g", force[6]);
     data[C_FORCE_Y] = g_strdup_printf ("%g", force[7]);
     data[C_FORCE_Z] = g_strdup_printf ("%g", force[8]);
     }
     }
  */
  
  gtk_clist_thaw (tbl);
}
