/*  COSMIX: general n_body simulator, version 0.4 Beta
    Copyright (C) 2001 Shan Mignot

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#include <stdlib.h>
#include <stdio.h>
#include <gtk/gtk.h>
#include <GL/gl.h>
#include <GL/glu.h>
#include <gtkgl/gtkglarea.h>
#include <libxml/tree.h>
#include <libxml/xmlmemory.h>

#include "define.h"
#include "functions.h"
#include "text.h"

/* Icons' creation macro */

GtkWidget *new_pixmap_widget (GdkWindow *window,
			      GdkColor *background,
			      gchar *filename)
{
  GtkWidget *wpixmap = NULL;
  GdkPixmap *pixmap = NULL;
  GdkBitmap *mask = NULL;
  
  pixmap = gdk_pixmap_create_from_xpm (window, &mask, background,
				       filename);
  wpixmap = gtk_pixmap_new (pixmap, mask);
  
  return (wpixmap);
  } 

GdkPixmap *new_pixmap (GdkWindow *window,
		       GdkColor *background,
		       gchar *filename)
{
  GdkPixmap *pixmap = NULL;
  GdkBitmap *mask = NULL;
  
  pixmap = gdk_pixmap_create_from_xpm (window, &mask, background,
				       filename);
  return (pixmap);
}

GdkBitmap *new_bitmap (GdkWindow *window,
		       GdkColor *background,
		       gchar *filename)
{
  GdkPixmap *pixmap = NULL;
  GdkBitmap *mask = NULL;
  
  pixmap = gdk_pixmap_create_from_xpm (window, &mask, background,
				       filename);
  return (mask);
}

/* -- Main structures' definitions -- */

void main_menu_create (GtkWidget *main_window,
		       GtkWidget *main_box)
{
  GtkWidget *main_menu_handle_box = gtk_handle_box_new ();
  GtkWidget *main_menu_bar = NULL;
  GtkItemFactory *main_menu_factory = NULL;
  GtkAccelGroup *main_menu_accel_group = gtk_accel_group_new ();
  GtkItemFactoryEntry main_menu_items[] = 
  {
    /* File Menu */
    { _MENU_FILE,                      NULL,       NULL,              0, "<Branch>"},
    { _MENU_FILE_TEAR,                 NULL,       NULL,              0,"<Tearoff>"},
    { _MENU_FILE_NEW,                  NULL,       NULL,              0, "<Branch>"},
    { _MENU_FILE_NEW_PROJECT,          NULL,       project_new,       0,       NULL},
    { _MENU_FILE_NEW_SEP1,             NULL,       NULL,              0,"<Separator>"},
    { _MENU_FILE_NEW_SYSTEM,   "<control>N",       system_new,        0,       NULL},
    { _MENU_FILE_NEW_SCR,              NULL,       script_new,        0,       NULL},
    { _MENU_FILE_LOAD,                 NULL,       NULL,              0,"<Branch>"},
    { _MENU_FILE_LOAD_PROJECT,         NULL,       project_load,      0,       NULL},
    { _MENU_FILE_LOAD_SEP1,            NULL,       NULL,              0,"<Separator>"},
    { _MENU_FILE_LOAD_SYSTEM,  "<control>L",       system_load,       0,       NULL},
    { _MENU_FILE_LOAD_SCR,             NULL,       script_load,       0,       NULL},
    { _MENU_FILE_LOAD_SEP2,            NULL,       NULL,              0,"<Separator>"},
    { _MENU_FILE_LOAD_OBJECTS,         NULL,       objects_load,      0,       NULL},
    { _MENU_FILE_LOAD_CNSTS,           NULL,       cnsts_load,        0,       NULL},
    { _MENU_FILE_RELOAD,               NULL,       reload,            0,       NULL},
    { _MENU_FILE_SEP1,                 NULL,       NULL,              0,"<Separator>"},
    { _MENU_FILE_RESET,        "<control>R",       reset,             0,       NULL},
    { _MENU_FILE_SAVE,                 NULL,       save,              0,       NULL},
    { _MENU_FILE_SAVE_AS,              NULL,       NULL,              0, "<Branch>"},
    { _MENU_FILE_SAVE_AS_PROJECT,      NULL,       project_save_as,   0,       NULL},
    { _MENU_FILE_SAVE_AS_SEP1,         NULL,       NULL,              0,"<Separator>"},
    { _MENU_FILE_SAVE_AS_SYSTEM, "<control>S",       system_save_as,    0,       NULL},
    { _MENU_FILE_SAVE_AS_SCR,          NULL,       script_save_as,    0,       NULL},
    { _MENU_FILE_SAVE_AS_SEP2,         NULL,       NULL,              0,"<Separator>"},
    { _MENU_FILE_SAVE_AS_TBL,          NULL,       tbl_save_as,       0,       NULL},
    { _MENU_FILE_SAVE_AS_ANIM,         NULL,       animation_save_as, 0,       NULL},
    { _MENU_FILE_SAVE_AS_SEP3,         NULL,       NULL,              0,"<Separator>"},
    { _MENU_FILE_SAVE_AS_OBJECTS,      NULL,       objects_save_as,   0,       NULL},
    { _MENU_FILE_SAVE_AS_CNSTS,        NULL,       cnsts_save_as,     0,       NULL},
    { _MENU_FILE_SEP2,                 NULL,       NULL,              0,"<Separator>"}, 
    { _MENU_FILE_PROPERTIES,           NULL,       properties,        0,       NULL}, 
    { _MENU_FILE_SEP3,                 NULL,       NULL,              0,"<Separator>"}, 
    { _MENU_FILE_QUIT,         "<control>Q",       quit,              0,       NULL}, 
    /* Edit menu */
    { _MENU_EDIT,                      NULL,       NULL,              0, "<Branch>"},
    { _MENU_EDIT_TEAR,                 NULL,       NULL,              0,"<Tearoff>"},
    /* Def menu */
    { _MENU_DEF,                       NULL,       NULL,              0, "<Branch>"},
    { _MENU_DEF_TEAR,                  NULL,       NULL,              0,"<Tearoff>"},
    /* Res menu */
    { _MENU_RES,                       NULL,       NULL,              0, "<Branch>"},
    { _MENU_RES_TEAR,                  NULL,       NULL,              0,"<Tearoff>"},
    /* Scr menu */
    { _MENU_SCR,                       NULL,       NULL,              0, "<Branch>"},
    { _MENU_SCR_TEAR,                  NULL,       NULL,              0,"<Tearoff>"},
    /* Tut menu */
    { _MENU_TUT,                       NULL,       NULL,              0, "<Branch>"},
    { _MENU_TUT_TEAR,                  NULL,       NULL,              0,"<Tearoff>"},
    /* Window menu */
    { _MENU_WINDOW,                    NULL,       NULL,              0, "<Branch>"},
    { _MENU_WINDOW_TEAR,               NULL,       NULL,              0,"<Tearoff>"},
    { _MENU_WINDOW_SHOW,               NULL,       NULL,              0, "<Branch>"},
    { _MENU_WINDOW_SHOW_CNT_PANEL,     NULL,       NULL,              0,       NULL},
    /* Help menu */
    { _MENU_HELP,                      NULL,       NULL,              0, "<Branch>"},
    { _MENU_HELP_TEAR,                 NULL,       NULL,              0,"<Tearoff>"},
    { _MENU_HELP_GETTING_STARTED,      NULL,       getting_started,   0,       NULL},
    { _MENU_HELP_ABOUT,                NULL,       about,             0,       NULL}
  };

  main_menu_factory = gtk_item_factory_new (GTK_TYPE_MENU_BAR,
					    "<main>",
					    main_menu_accel_group);
  gtk_item_factory_create_items (main_menu_factory,
				 sizeof (main_menu_items) /
				 sizeof (main_menu_items[0]),
				 main_menu_items, NULL);

  /* Use gtk_widget_set_sensitive to disable unavailable menu items. */
  gtk_widget_set_sensitive 
    (GTK_WIDGET (gtk_item_factory_get_item (main_menu_factory,
					    MENU_FILE_RELOAD)), FALSE);
  gtk_widget_set_sensitive 
    (GTK_WIDGET (gtk_item_factory_get_item (main_menu_factory,
					    MENU_FILE_LOAD_PROJECT)), FALSE);
  gtk_widget_set_sensitive 
    (GTK_WIDGET (gtk_item_factory_get_item (main_menu_factory,
					    MENU_FILE_LOAD_OBJECTS)), FALSE);
  gtk_widget_set_sensitive 
    (GTK_WIDGET (gtk_item_factory_get_item (main_menu_factory,
					    MENU_FILE_LOAD_CNSTS)), FALSE);
  gtk_widget_set_sensitive 
    (GTK_WIDGET (gtk_item_factory_get_item (main_menu_factory,
					    MENU_FILE_SAVE)), FALSE);
  gtk_widget_set_sensitive 
    (GTK_WIDGET (gtk_item_factory_get_item (main_menu_factory,
					    MENU_FILE_SAVE_AS_PROJECT)), FALSE);
  gtk_widget_set_sensitive 
    (GTK_WIDGET (gtk_item_factory_get_item (main_menu_factory,
					    MENU_FILE_SAVE_AS_ANIM)), FALSE);
  gtk_widget_set_sensitive 
    (GTK_WIDGET (gtk_item_factory_get_item (main_menu_factory,
					    MENU_FILE_SAVE_AS_OBJECTS)), FALSE);
  gtk_widget_set_sensitive 
    (GTK_WIDGET (gtk_item_factory_get_item (main_menu_factory,
					    MENU_FILE_SAVE_AS_CNSTS)), FALSE);
  gtk_widget_set_sensitive 
    (GTK_WIDGET (gtk_item_factory_get_item (main_menu_factory,
					    MENU_FILE_PROPERTIES)), FALSE);
  gtk_widget_set_sensitive 
    (GTK_WIDGET (gtk_item_factory_get_item (main_menu_factory,
					    MENU_EDIT)), FALSE);
  gtk_widget_set_sensitive 
    (GTK_WIDGET (gtk_item_factory_get_item (main_menu_factory,
					    MENU_DEF)), FALSE);
  gtk_widget_set_sensitive 
    (GTK_WIDGET (gtk_item_factory_get_item (main_menu_factory,
					    MENU_RES)), FALSE);
  gtk_widget_set_sensitive 
    (GTK_WIDGET (gtk_item_factory_get_item (main_menu_factory,
					    MENU_SCR)), FALSE);
  gtk_widget_set_sensitive 
    (GTK_WIDGET (gtk_item_factory_get_item (main_menu_factory,
					    MENU_TUT)), FALSE);

  /* Setting up the callback functions */
  gtk_signal_connect
    (GTK_OBJECT 
     (gtk_item_factory_get_widget (main_menu_factory, MENU_WINDOW_SHOW_CNT_PANEL)),
     "activate", GTK_SIGNAL_FUNC (window_show),
     GINT_TO_POINTER (CONTROL_PANEL));
					      
  gtk_window_add_accel_group (GTK_WINDOW (main_window), 
			      main_menu_accel_group);
  main_menu_bar = gtk_item_factory_get_widget (main_menu_factory,
					       "<main>");

  /* Packing it all */
  gtk_box_pack_start (GTK_BOX (main_box), main_menu_handle_box,
		      FALSE, FALSE, NO_PADDING);
  gtk_container_add (GTK_CONTAINER (main_menu_handle_box),
				    main_menu_bar);
  gtk_widget_show_all (main_menu_handle_box);
}
 
void main_toolbar_create (GtkWidget *main_window,
			  GtkWidget *main_box,
			  GtkWidget **main_toolbar)
{
  GtkWidget *main_toolbar_handle_box = gtk_handle_box_new ();
  
  *main_toolbar = gtk_toolbar_new (GTK_ORIENTATION_HORIZONTAL,
				   GTK_TOOLBAR_ICONS);
  gtk_box_pack_start (GTK_BOX (main_box), main_toolbar_handle_box,
		      FALSE, FALSE, NO_PADDING);
  gtk_container_add (GTK_CONTAINER (main_toolbar_handle_box),
		     *main_toolbar);
		     
  /* Icons on the toolbar */
  gtk_toolbar_append_item (GTK_TOOLBAR (*main_toolbar), NULL,
			   TOOLTIP_NEW_PROJECT, TOOLTIP_NEW_PROJECT_LONG, 
			   new_pixmap_widget (main_window->window,
					      &main_window->style->
					      bg[GTK_STATE_NORMAL],
					      ICON_NEW_FILE),
			   GTK_SIGNAL_FUNC (project_new), NULL);
  gtk_toolbar_append_item (GTK_TOOLBAR (*main_toolbar), NULL,
			   TOOLTIP_LOAD, TOOLTIP_LOAD_LONG, 
			   new_pixmap_widget (main_window->window,
					      &main_window->style->
					      bg[GTK_STATE_NORMAL],
					      ICON_OPEN_FILE),
			   GTK_SIGNAL_FUNC (system_load), NULL);
  gtk_toolbar_append_item (GTK_TOOLBAR (*main_toolbar), NULL,
			   TOOLTIP_SAVE, TOOLTIP_SAVE_LONG, 
			   new_pixmap_widget (main_window->window,
					      &main_window->style->
					      bg[GTK_STATE_NORMAL],
					      ICON_SAVE_FILE),
			   GTK_SIGNAL_FUNC (system_save_as), NULL);
  
  gtk_widget_show_all (main_toolbar_handle_box);
}

void main_notebook_create (GtkWidget *main_box,
			   GtkTooltips *main_tooltips)
{
  gint i;
  gchar *tbl_column_names[N_TABLE_COLUMNS]
    =  {T_OBJ, T_T, T_X, T_Y, T_Z, T_VX, T_VY, T_VZ,
	T_ENERGY, T_K_ENERGY, T_G_ENERGY};
  /* T_G, T_FORCE_X, T_FORCE_Y, T_FORCE_Z,
     T_K_MOMENTUM_X, T_K_MOMENTUM_Y, T_K_MOMENTUM_Z}; */
  GtkWidget *notebook;
  GtkWidget *paned_windows[3] = {NULL, NULL, NULL};
  GtkWidget *label = NULL;
  /*  GtkWidget *event_box = NULL; */
  GtkWidget *frames[4] = {NULL, NULL, NULL, NULL};
  GtkWidget *scrolledwindow = NULL;
  GtkWidget *hbox = gtk_hbox_new (FALSE, NO_PADDING);
  GtkWidget *vscrollbar = NULL;
  GtkWidget *scr = NULL;

  notebook = gtk_notebook_new ();
  global_widgets[NOTEBOOK].exists = TRUE;
  global_widgets[NOTEBOOK].widget = notebook;
  gtk_notebook_set_tab_pos (GTK_NOTEBOOK (notebook), GTK_POS_LEFT);
  gtk_notebook_set_scrollable (GTK_NOTEBOOK (notebook), TRUE);
 
  /* Def page creation */
  /* The paned windows */
  paned_windows[MAIN_V] = gtk_vpaned_new();
  paned_windows[SUB_H1] = gtk_hpaned_new();
  paned_windows[SUB_H2] = gtk_hpaned_new();
  for (i = 0; i < 3; i++)
    {
      /* The horizontal handle is hardly visible, change its size... */
      gtk_paned_set_handle_size (GTK_PANED (paned_windows[i]),
				 HANDLE_SIZE);
      gtk_paned_set_gutter_size (GTK_PANED (paned_windows[i]),
				 GUTTER_SIZE);
      gtk_paned_set_position (GTK_PANED (paned_windows[i]),
			      PANED_POSITION);
    }
  /* The frames and GL areas inside each paned window */
  for (i = 0; i < 4; i++) 
    {
      frames[i] = gtk_frame_new (NULL);
      gl_area_set (frames[i], DEF_GL_X + i);
      if (i == 0)
	{
	  global_widgets[DEF_GL_X].exists = TRUE;
	  global_widgets[DEF_GL_X].widget =
	    GTK_WIDGET (gtk_container_children (GTK_CONTAINER (frames[0]))->data);
	}
    }
  /* Reference as global widgets */
  global_widgets[DEF_GL_Y].exists = TRUE;
  global_widgets[DEF_GL_Y].widget =
    GTK_WIDGET (gtk_container_children (GTK_CONTAINER (frames[1]))->data);
  global_widgets[DEF_GL_Z].exists = TRUE;
  global_widgets[DEF_GL_Z].widget =
    GTK_WIDGET (gtk_container_children (GTK_CONTAINER (frames[2]))->data);
  global_widgets[DEF_GL_USER].exists = TRUE;
  global_widgets[DEF_GL_USER].widget =
    GTK_WIDGET (gtk_container_children (GTK_CONTAINER (frames[3]))->data);

  gtk_paned_add1 (GTK_PANED (paned_windows[MAIN_V]), 
		  paned_windows[SUB_H1]);
  gtk_paned_add2 (GTK_PANED (paned_windows[MAIN_V]), 
		  paned_windows[SUB_H2]);
  gtk_paned_add1 (GTK_PANED (paned_windows[SUB_H1]), 
		  frames[TOP_LEFT]);
  gtk_paned_add2 (GTK_PANED (paned_windows[SUB_H1]), 
		  frames[TOP_RIGHT]);
  gtk_paned_add1 (GTK_PANED (paned_windows[SUB_H2]), 
		  frames[BTM_LEFT]);
  gtk_paned_add2 (GTK_PANED (paned_windows[SUB_H2]), 
		  frames[BTM_RIGHT]);
  /* Resize and shrink policy */
  for (i = 0; i < 2; i++)
    {
      gtk_widget_ref (frames [2*i]);
      gtk_container_remove (GTK_CONTAINER (paned_windows[i+1]),
			    frames[2*i]);
      gtk_paned_pack1 (GTK_PANED (paned_windows[i+1]), frames[2*i],
		       TRUE, TRUE);
      gtk_widget_unref (frames[2*i]);
    }
  gtk_widget_ref (paned_windows[SUB_H1]);
  gtk_container_remove (GTK_CONTAINER (paned_windows[MAIN_V]), 
			paned_windows[SUB_H1]);
  gtk_paned_pack1 (GTK_PANED (paned_windows[MAIN_V]),
			      paned_windows[SUB_H1], TRUE, TRUE);
  gtk_widget_unref (paned_windows[SUB_H1]);
  /* Append the def page */
  label = gtk_label_new (TAB_DEF);
  /* For setting tooltips
     event_box = gtk_event_box_new (); 
     gtk_tooltips_set_tip (main_tooltips, event_box,
     TOOLTIP_DEF_TAB, TOOLTIP_DEF_TAB_LONG);
     gtk_container_add (GTK_CONTAINER (event_box), label);
     gtk_notebook_append_page (GTK_NOTEBOOK (notebook),
     paned_windows[MAIN_V], event_box);
     gtk_notebook_set_tab_label_packing (GTK_NOTEBOOK (notebook),
     paned_windows[MAIN_V],
     TRUE, TRUE, GTK_PACK_START);
     gtk_widget_show (label); */
  gtk_notebook_append_page (GTK_NOTEBOOK (notebook),
			    paned_windows[MAIN_V], label); 
  gtk_widget_show (paned_windows[MAIN_V]);
  
  /* Grp page creation */
  /* The paned windows */
  paned_windows[MAIN_V] = gtk_vpaned_new();
  paned_windows[SUB_H1] = gtk_hpaned_new();
  paned_windows[SUB_H2] = gtk_hpaned_new();
  for (i = 0; i < 3; i++)
    {
      gtk_paned_set_handle_size (GTK_PANED (paned_windows[i]),
				 HANDLE_SIZE);
      gtk_paned_set_gutter_size (GTK_PANED (paned_windows[i]),
				 GUTTER_SIZE);
      gtk_paned_set_position (GTK_PANED (paned_windows[i]),
			      PANED_POSITION);
    }
  /* The frames and drawing areas inside each paned window */
  for (i = 0; i < 4; i++) 
    {
      frames[i] = gtk_frame_new (NULL);
      gl_area_set (frames[i], GRP_GL_X + i);
      if (i == 0)
	{
	  global_widgets[GRP_GL_X].exists = TRUE;
	  global_widgets[GRP_GL_X].widget =
	    GTK_WIDGET (gtk_container_children (GTK_CONTAINER (frames[0]))->data);
	}
    }
  /* Reference as global widgets and realize */
  global_widgets[GRP_GL_Y].exists = TRUE;
  global_widgets[GRP_GL_Y].widget =
    GTK_WIDGET (gtk_container_children (GTK_CONTAINER (frames[1]))->data);
  global_widgets[GRP_GL_Z].exists = TRUE;
  global_widgets[GRP_GL_Z].widget =
    GTK_WIDGET (gtk_container_children (GTK_CONTAINER (frames[2]))->data);
  global_widgets[GRP_GL_USER].exists = TRUE;
  global_widgets[GRP_GL_USER].widget =
    GTK_WIDGET (gtk_container_children (GTK_CONTAINER (frames[3]))->data);

  gtk_paned_add1 (GTK_PANED (paned_windows[MAIN_V]), 
		  paned_windows[SUB_H1]);
  gtk_paned_add2 (GTK_PANED (paned_windows[MAIN_V]), 
		  paned_windows[SUB_H2]);
  gtk_paned_add1 (GTK_PANED (paned_windows[SUB_H1]), 
		  frames[TOP_LEFT]);
  gtk_paned_add2 (GTK_PANED (paned_windows[SUB_H1]), 
		  frames[TOP_RIGHT]);
  gtk_paned_add1 (GTK_PANED (paned_windows[SUB_H2]), 
		  frames[BTM_LEFT]);
  gtk_paned_add2 (GTK_PANED (paned_windows[SUB_H2]), 
		  frames[BTM_RIGHT]);
  /* Resize and shrink policy */
  for (i = 0; i < 2; i++)
    {
      gtk_widget_ref (frames [2*i]);
      gtk_container_remove (GTK_CONTAINER (paned_windows[i+1]),
			    frames[2*i]);
      gtk_paned_pack1 (GTK_PANED (paned_windows[i+1]), frames[2*i],
		       TRUE, TRUE);
      gtk_widget_unref (frames[2*i]);
    }
  gtk_widget_ref (paned_windows[SUB_H1]);
  gtk_container_remove (GTK_CONTAINER (paned_windows[MAIN_V]), 
			paned_windows[SUB_H1]);
  gtk_paned_pack1 (GTK_PANED (paned_windows[MAIN_V]),
			      paned_windows[SUB_H1], TRUE, TRUE);
  gtk_widget_unref (paned_windows[SUB_H1]);
  /* Append the grp page */
  label = gtk_label_new (TAB_GRP);
  gtk_notebook_append_page (GTK_NOTEBOOK (notebook),
			    paned_windows[MAIN_V], label);
/*  gtk_notebook_set_tab_label_packing (GTK_NOTEBOOK (notebook),
    paned_windows[MAIN_V], TRUE, TRUE, GTK_PACK_START); */
  gtk_widget_show (paned_windows[MAIN_V]);
  
  /* Tbl page creation */
  scrolledwindow = gtk_scrolled_window_new (NULL, NULL);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW
				  (scrolledwindow), 
				  GTK_POLICY_AUTOMATIC,
				  GTK_POLICY_AUTOMATIC);
  global_widgets[TBL].widget = gtk_clist_new_with_titles (N_TABLE_COLUMNS,
							  tbl_column_names);
  global_widgets[TBL].exists = TRUE;
  gtk_clist_column_titles_active (GTK_CLIST (global_widgets[TBL].widget));
  gtk_signal_connect (GTK_OBJECT (global_widgets[TBL].widget),
		      "click_column", GTK_SIGNAL_FUNC (tbl_hide_column), NULL);
  gtk_clist_set_shadow_type (GTK_CLIST (global_widgets[TBL].widget),
			     GTK_SHADOW_NONE);
  gtk_clist_set_selection_mode (GTK_CLIST (global_widgets[TBL].widget),
				GTK_SELECTION_MULTIPLE);
  gtk_container_add (GTK_CONTAINER (scrolledwindow), global_widgets[TBL].widget);
  for (i = 0; i < N_TABLE_COLUMNS; i++)
    {
      gtk_clist_set_column_auto_resize (GTK_CLIST (global_widgets[TBL].widget),
					i, TRUE);
      gtk_clist_set_column_justification (GTK_CLIST (global_widgets[TBL].widget),
					  i, GTK_JUSTIFY_RIGHT);
    }				  
  /* Append the Tbl page */
  label = gtk_label_new (TAB_TBL);
  gtk_notebook_append_page (GTK_NOTEBOOK (notebook),
			    scrolledwindow, label);
/*  gtk_notebook_set_tab_label_packing (GTK_NOTEBOOK (notebook),
    scrolledwindow, TRUE, TRUE, GTK_PACK_START); */
  gtk_widget_show (scrolledwindow);

  /* Scr page creation */
  scr = gtk_text_new (NULL, NULL);
  gtk_text_set_editable (GTK_TEXT (scr), TRUE);
  gtk_text_set_word_wrap (GTK_TEXT (scr), TRUE);
  vscrollbar = gtk_vscrollbar_new (GTK_TEXT (scr)->vadj);
  /* Register as a global_widget */
  global_widgets[SCRIPT].exists = TRUE;
  global_widgets[SCRIPT].widget = scr;
  /* Append the Scr page */
  gtk_box_pack_start (GTK_BOX (hbox), scr, TRUE, TRUE, NO_PADDING);
  gtk_box_pack_start (GTK_BOX (hbox), vscrollbar,
		      FALSE, FALSE, NO_PADDING);  
  label = gtk_label_new (TAB_SCR);
  gtk_notebook_append_page (GTK_NOTEBOOK (notebook), hbox, label);
/*  gtk_notebook_set_tab_label_packing (GTK_NOTEBOOK (notebook),
    hbox, TRUE, TRUE, GTK_PACK_START); */

  /* Insert the notebook */
  gtk_box_pack_start (GTK_BOX (main_box), notebook, TRUE, TRUE,
		      NO_PADDING);
  gtk_widget_show_all (notebook);
}

void main_status_bar_create (GtkWidget *main_box,
			     GtkWidget **status_bar,
			     gint *context_id)
{
  *status_bar = gtk_statusbar_new ();
  *context_id = gtk_statusbar_get_context_id (GTK_STATUSBAR (*status_bar),
					      "Main Status");

  /* Pack it all */
  gtk_box_pack_start (GTK_BOX (main_box), *status_bar,
		      FALSE, TRUE, NO_PADDING);
  gtk_widget_show (*status_bar);
}

/* -- Control panel's definition */

void tree_controls_create (GtkWidget *control_panel_window,
			   GtkWidget *def_vbox)
{
  GtkWidget *handle_box = gtk_handle_box_new ();
  GtkWidget *hbox = gtk_hbox_new (FALSE, NO_PADDING);
  GtkWidget *scrolled_window = gtk_scrolled_window_new (NULL, NULL);
  GtkWidget *obj_toolbar = gtk_toolbar_new (GTK_ORIENTATION_VERTICAL,
					    GTK_TOOLBAR_ICONS);
  GtkCList *system_list = NULL;
  GtkCTree *system_tree = NULL;
  gchar *tree_titles[N_SYS_COL] = {TREE, CHKLST, STATUS,
				   NAME, COMMENT};
  gint i;

  gtk_container_add (GTK_CONTAINER (handle_box), hbox);
  gtk_box_pack_start (GTK_BOX (def_vbox), handle_box,
		      TRUE, TRUE, NO_PADDING);
  gtk_widget_set_usize (scrolled_window, TREE_WINDOW_HSIZE, TREE_WINDOW_VSIZE);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window),
				  GTK_POLICY_AUTOMATIC,
				  GTK_POLICY_AUTOMATIC);
  gtk_box_pack_start (GTK_BOX (hbox), scrolled_window,
		      TRUE, TRUE, NO_PADDING);

  /* System tree */

  global_widgets[SYSTEM_TREE].widget = 
    gtk_ctree_new_with_titles (N_SYS_COL, N_TREE_COL, tree_titles);
  global_widgets[SYSTEM_TREE].exists = TRUE;
  system_tree = GTK_CTREE (global_widgets[SYSTEM_TREE].widget);
  system_list = GTK_CLIST (system_tree);
  gtk_clist_set_selection_mode (system_list, GTK_SELECTION_BROWSE);
  for (i = 0; i < N_SYS_COL - 1; i++)
    gtk_clist_set_column_justification (system_list, i,
					GTK_JUSTIFY_CENTER);
  gtk_clist_set_column_justification (system_list, N_SYS_COL - 1,
				      GTK_JUSTIFY_LEFT);
  gtk_clist_column_title_passive (system_list, 0); 
  gtk_clist_column_title_passive (system_list, 1);
  gtk_clist_set_column_auto_resize (system_list, 0, TRUE); 
  gtk_clist_set_column_auto_resize (system_list, 3, TRUE);
  gtk_clist_set_column_auto_resize (system_list, 4, TRUE);
  gtk_clist_set_column_width (system_list, 0, 
			      gtk_clist_optimal_column_width 
			      (system_list, 0));
  gtk_clist_set_column_width (system_list, 1, 
			      gtk_clist_optimal_column_width 
			      (system_list, 1));
  gtk_clist_set_column_width (system_list, 2, 
			      gtk_clist_optimal_column_width 
			      (system_list, 2));
  gtk_ctree_set_line_style (system_tree, GTK_CTREE_LINES_DOTTED);
  gtk_container_add (GTK_CONTAINER (scrolled_window),
		     GTK_WIDGET (system_tree));
  gtk_signal_connect (GTK_OBJECT (system_list), "click_column",
		      GTK_SIGNAL_FUNC (tree_utils), NULL);
  gtk_signal_connect (GTK_OBJECT (system_list), "button-press-event",
		      GTK_SIGNAL_FUNC (tree_popup_menu), NULL);
  gtk_signal_connect (GTK_OBJECT (system_list), "select-row",
		      GTK_SIGNAL_FUNC (display_selected_row), NULL);
  gtk_container_set_focus_vadjustment (GTK_CONTAINER (system_tree),
				       gtk_scrolled_window_get_vadjustment
				       (GTK_SCROLLED_WINDOW (scrolled_window)));
  /*  gtk_container_set_focus_hadjustment (GTK_CONTAINER (system_tree),
      gtk_scrolled_window_get_hadjustment
      (GTK_SCROLLED_WINDOW (scrolled_window))); */
  
  /* Physical objects */
  gtk_toolbar_append_item (GTK_TOOLBAR (obj_toolbar), NULL,
			   TOOLTIP_PT_OBJ, TOOLTIP_PT_OBJ_LONG,
			   new_pixmap_widget
			   (control_panel_window->window,
			    &control_panel_window->style->
			    bg[GTK_STATE_NORMAL],
			    ICON_PT_OBJ_2),
			   GTK_SIGNAL_FUNC (pt_obj_new), NULL);
  gtk_box_pack_start (GTK_BOX (hbox), obj_toolbar,
		      FALSE, FALSE, NO_PADDING);
  gtk_widget_show_all (def_vbox);
}

void phy_controls_create (GtkWidget *def_vbox)
{
  GtkWidget *handle_box = gtk_handle_box_new ();
  GtkWidget *hbox = gtk_hbox_new (FALSE, NO_PADDING);
  GtkWidget *hbox2 = gtk_hbox_new (FALSE, NO_PADDING);
  GtkWidget *vbox1 = gtk_vbox_new (FALSE, NO_PADDING);
  GtkWidget *vbox2 = gtk_vbox_new (FALSE, NO_PADDING); 
  GtkWidget *name_label = NULL, *label = NULL;
  GtkWidget *mass_entry = NULL, *dimension_entry = NULL;
  TwoWidgets *mass_widgets = NULL, *dimension_widgets = NULL;
  gint id = DEF_PHY;

  mass_widgets = g_new0 (TwoWidgets, 1);
  dimension_widgets = g_new0 (TwoWidgets, 1);

  /* The name of the active object */
  /* This field should include the nature of the model associated to 
     the object when there will be several possible ones */
  name_label = gtk_label_new (NULL);
  gtk_box_pack_start (GTK_BOX (vbox1), name_label, FALSE, FALSE, NO_PADDING);
  /* Its physical properties */
  /* Mass */
  label = gtk_label_new (MASS);
  gtk_box_pack_start (GTK_BOX (hbox), label, TRUE, TRUE, NO_PADDING);
  label = gtk_label_new (DIMENSION);
  gtk_box_pack_start (GTK_BOX (hbox), label, TRUE, TRUE, NO_PADDING);
  gtk_box_pack_start (GTK_BOX (vbox1), hbox, FALSE, FALSE, NO_PADDING);
  /* Filter the input to numbers (mantissa and exponent) */
  /* Retrieve the entry by connecting the changed signal */ 
  hbox = gtk_hbox_new (FALSE, SPACING);
  mass_entry = gtk_entry_new ();
  gtk_widget_set_usize (mass_entry, ENTRY_HSIZE, ENTRY_VSIZE);
  gtk_signal_connect(GTK_OBJECT(mass_entry), "insert_text",
		     GTK_SIGNAL_FUNC(inserted_text_handler),
		     NULL);
  gtk_signal_connect (GTK_OBJECT (mass_entry), "activate",
		      GTK_SIGNAL_FUNC (changed_text_handler),
		      GINT_TO_POINTER (E_MASS));
  mass_widgets->widget1 = gtk_button_new_with_label (MENU_MEASURE_MEAN);
  mass_widgets->widget2 = gtk_label_new (NULL); 

  /* Disabled for now because the callbacks have not been written */
  gtk_widget_set_sensitive (GTK_WIDGET (mass_widgets->widget1), FALSE);

  gtk_signal_connect (GTK_OBJECT (mass_widgets->widget1), "clicked",
		      GTK_SIGNAL_FUNC (measure_mode), mass_widgets);
  gtk_box_pack_start (GTK_BOX (vbox2), mass_entry, TRUE, TRUE, NO_PADDING); 
  gtk_box_pack_start (GTK_BOX (hbox2), mass_widgets->widget1,
		      FALSE, FALSE, NO_PADDING); 
  gtk_box_pack_start (GTK_BOX (hbox2), mass_widgets->widget2,
		      FALSE, FALSE, NO_PADDING); 
  gtk_box_pack_start (GTK_BOX (vbox2), hbox2, FALSE, FALSE, NO_PADDING); 
  gtk_box_pack_start (GTK_BOX (hbox), vbox2, TRUE, TRUE, NO_PADDING);
  /* Dimension */
  vbox2 = gtk_vbox_new (FALSE, NO_PADDING);
  hbox2 = gtk_hbox_new (FALSE, NO_PADDING);
  /* Filter the input to numbers (mantissa and exponent) */  
  /* Retrieve the entry by connecting the changed signal */
  dimension_entry = gtk_entry_new ();
  gtk_widget_set_usize (dimension_entry, ENTRY_HSIZE, ENTRY_VSIZE);
  gtk_signal_connect(GTK_OBJECT(dimension_entry), "insert_text",
		     GTK_SIGNAL_FUNC(inserted_text_handler),
		     NULL);
  gtk_signal_connect (GTK_OBJECT (dimension_entry), "activate",
		      GTK_SIGNAL_FUNC (changed_text_handler),
		      GINT_TO_POINTER (E_DIMENSION));

  /* Disabled for now because the callbacks have not been written */
  gtk_widget_set_sensitive (GTK_WIDGET (dimension_entry), FALSE);

  dimension_widgets->widget1 = gtk_button_new_with_label (MENU_MEASURE_MEAN);
  dimension_widgets->widget2 = gtk_label_new (NULL);


  /* Disabled for now because the callbacks have not been written */
  gtk_widget_set_sensitive (GTK_WIDGET (dimension_widgets->widget1), FALSE);

  gtk_signal_connect (GTK_OBJECT (dimension_widgets->widget1), "clicked",
		      GTK_SIGNAL_FUNC (measure_mode),
		      dimension_widgets);
  gtk_box_pack_start (GTK_BOX (vbox2), dimension_entry,
		      TRUE, TRUE, NO_PADDING); 
  gtk_box_pack_start (GTK_BOX (hbox2), dimension_widgets->widget1,
		      FALSE, FALSE, NO_PADDING); 
  gtk_box_pack_start (GTK_BOX (hbox2), dimension_widgets->widget2,
		      FALSE, FALSE, NO_PADDING); 
  gtk_box_pack_start (GTK_BOX (vbox2), hbox2, FALSE, FALSE, NO_PADDING); 
  gtk_box_pack_start (GTK_BOX (hbox), vbox2, TRUE, TRUE, NO_PADDING);
  gtk_box_pack_start (GTK_BOX (vbox1), hbox, FALSE, FALSE, NO_PADDING);
  gtk_container_add (GTK_CONTAINER (handle_box), vbox1);
  global_widgets[id].widget = vbox1;
  global_widgets[id].exists = TRUE;
  gtk_box_pack_start (GTK_BOX (def_vbox), handle_box, 
		      FALSE, FALSE, NO_PADDING);
  /* This also displays the handle box, it should not */
  gtk_widget_show_all (vbox1);
}

void ki_controls_create (GtkWidget *def_vbox)
{
  GtkWidget *handle_box = gtk_handle_box_new ();
  GtkWidget *name_label = NULL, *label = NULL;
  GtkWidget *position_entry[3] = {NULL, NULL, NULL};
  GtkWidget *speed_entry[3] = {NULL, NULL, NULL};
  GtkWidget *table = gtk_table_new (6, 4, FALSE);
  TwoWidgets *position_widgets = NULL, *speed_widgets = NULL;
  gint id = DEF_KI;
  gint i;
  gchar *position_text[3] = {T_X, T_Y, T_Z};
  gchar *speed_text[3] = {T_VX, T_VY, T_VZ};
  

  position_widgets = g_new0 (TwoWidgets, 1);
  speed_widgets = g_new0 (TwoWidgets, 1);

  /* The name of the active object */
  /* This field should include the nature of the model associated to 
     the object when there will be several possible ones */
  name_label = gtk_label_new (NULL);
  gtk_table_attach (GTK_TABLE (table), name_label, 0, 4, 0, 1,
		    GTK_SHRINK, GTK_SHRINK, NO_PADDING, NO_PADDING);
  /* Its  kinetics */
  /* Position */
  label = gtk_label_new (POSITION);
  gtk_table_attach (GTK_TABLE (table), label, 1, 2, 1, 2,
		    GTK_SHRINK, GTK_SHRINK, NO_PADDING, NO_PADDING);
  label = gtk_label_new (SPEED);
  gtk_table_attach (GTK_TABLE (table), label, 3, 4, 1, 2,
		    GTK_SHRINK, GTK_SHRINK, NO_PADDING, NO_PADDING);
  /* Filter the input to numbers (mantissa and exponent) */
  /* Retrieve the entry by connecting the changed signal */ 
  for (i = 0; i < 3; i++)
    {
      label = gtk_label_new (position_text[i]);
      position_entry[i] = gtk_entry_new ();
      gtk_widget_set_usize (position_entry[i], ENTRY_HSIZE, ENTRY_VSIZE);
      gtk_signal_connect (GTK_OBJECT(position_entry[i]), "insert_text",
			 GTK_SIGNAL_FUNC(inserted_text_handler),
			 NULL);
      gtk_table_attach (GTK_TABLE (table), label, 0, 1, i+2, i+3,
			GTK_SHRINK, GTK_SHRINK, NO_PADDING, NO_PADDING);
      gtk_table_attach (GTK_TABLE (table), position_entry[i], 1, 2, i+2, i+3,
			GTK_FILL | GTK_SHRINK | GTK_EXPAND,
			GTK_FILL | GTK_SHRINK | GTK_EXPAND,
			NO_PADDING, NO_PADDING);
    }     
  gtk_signal_connect (GTK_OBJECT (position_entry[0]), "activate",
		      GTK_SIGNAL_FUNC (changed_text_handler),
		      GINT_TO_POINTER (E_POSITION_X)); 
  gtk_signal_connect (GTK_OBJECT (position_entry[1]), "activate",
		      GTK_SIGNAL_FUNC (changed_text_handler),
		      GINT_TO_POINTER (E_POSITION_Y));
  gtk_signal_connect (GTK_OBJECT (position_entry[2]), "activate",
		      GTK_SIGNAL_FUNC (changed_text_handler),
		      GINT_TO_POINTER (E_POSITION_Z));
  /* Speed */
  /* Retrieve the entry by connecting the changed signal */
  for (i = 0; i < 3; i++)
    {
      label = gtk_label_new (speed_text[i]);
      speed_entry[i] = gtk_entry_new ();
      gtk_widget_set_usize (speed_entry[i], ENTRY_HSIZE, ENTRY_VSIZE);
      gtk_signal_connect (GTK_OBJECT(speed_entry[i]), "insert_text",
			  GTK_SIGNAL_FUNC(inserted_text_handler),
			  NULL);
      gtk_table_attach (GTK_TABLE (table), label, 2, 3, i+2, i+3,
			GTK_SHRINK, GTK_SHRINK, NO_PADDING, NO_PADDING);
      gtk_table_attach (GTK_TABLE (table), speed_entry[i], 3, 4, i+2, i+3,
			GTK_FILL | GTK_SHRINK | GTK_EXPAND,
			GTK_FILL | GTK_SHRINK | GTK_EXPAND,
			NO_PADDING, NO_PADDING);
    }
  gtk_signal_connect (GTK_OBJECT (speed_entry[0]), "activate",
		      GTK_SIGNAL_FUNC (changed_text_handler),
		      GINT_TO_POINTER (E_SPEED_X));
  gtk_signal_connect (GTK_OBJECT (speed_entry[1]), "activate",
		      GTK_SIGNAL_FUNC (changed_text_handler),
		      GINT_TO_POINTER (E_SPEED_Y));
  gtk_signal_connect (GTK_OBJECT (speed_entry[2]), "activate",
		      GTK_SIGNAL_FUNC (changed_text_handler),
		      GINT_TO_POINTER (E_SPEED_Z));
  /* Measures */
  position_widgets->widget1 = gtk_button_new_with_label (MENU_MEASURE_MEAN);
  position_widgets->widget2 = gtk_label_new (NULL); 

  /* Disabled for now because the callbacks have not been written */
  gtk_widget_set_sensitive (GTK_WIDGET (position_widgets->widget1), FALSE);

  gtk_signal_connect (GTK_OBJECT (position_widgets->widget1), "clicked",
		      GTK_SIGNAL_FUNC (measure_mode),
		      position_widgets);
  gtk_table_attach (GTK_TABLE (table), position_widgets->widget1, 0, 1, 5, 6,
		    GTK_SHRINK, GTK_SHRINK, NO_PADDING, NO_PADDING);
  gtk_table_attach (GTK_TABLE (table), position_widgets->widget2, 1, 2, 5, 6,
		    GTK_SHRINK, GTK_SHRINK, NO_PADDING, NO_PADDING);
  speed_widgets->widget1 = gtk_button_new_with_label (MENU_MEASURE_MEAN);
  speed_widgets->widget2 = gtk_label_new (NULL);

  /* Disabled for now because the callbacks have not been written */
  gtk_widget_set_sensitive (GTK_WIDGET (speed_widgets->widget1), FALSE);

  gtk_signal_connect (GTK_OBJECT (speed_widgets->widget1), "clicked",
		      GTK_SIGNAL_FUNC (measure_mode),
		      speed_widgets);
  gtk_table_attach (GTK_TABLE (table), speed_widgets->widget1, 2, 3, 5, 6,
		    GTK_SHRINK, GTK_SHRINK, NO_PADDING, NO_PADDING);
  gtk_table_attach (GTK_TABLE (table), speed_widgets->widget2, 3, 4, 5, 6,
		    GTK_SHRINK, GTK_SHRINK, NO_PADDING, NO_PADDING);
  /* Pack it all */
  gtk_container_add (GTK_CONTAINER (handle_box), table);
  global_widgets[id].widget = table;
  global_widgets[id].exists = TRUE;
  gtk_box_pack_start (GTK_BOX (def_vbox), handle_box, 
		      FALSE, FALSE, NO_PADDING);
  gtk_widget_show_all (table);
}

void simu_controls_create (GtkWidget *def_vbox)
{
   GtkWidget *handle_box = gtk_handle_box_new ();
   GtkWidget *table = gtk_table_new (5, 3, FALSE);
   GtkWidget *start_entry = gtk_entry_new ();
   GtkWidget *stop_entry = gtk_entry_new ();
   GtkWidget *n_step_entry = gtk_entry_new ();
   GtkWidget *raccuracy_entry = gtk_entry_new ();
   GtkWidget *aaccuracy_entry = gtk_entry_new ();
   GtkWidget *simulate_button = gtk_button_new_with_label (SIMULATE);
   TwoWidgets *measure_widgets = NULL;
   GtkWidget *label = NULL;
   gint i;
   gchar *text[5] = {START, STOP, N_STEP, RACCURACY, AACCURACY};
   
   measure_widgets = g_new0 (TwoWidgets, 1);
   measure_widgets->widget1 = gtk_button_new_with_label (DURATION);

  /* Disabled for now because the callbacks have not been written */
   gtk_widget_set_sensitive (GTK_WIDGET (measure_widgets->widget1), FALSE);

   measure_widgets->widget2 = gtk_label_new (NULL);
   simu_parameters = g_new0 (Simulation, 1);
   
   /* Setting up the labels */
   for (i = 0; i < 5; i++)
   {
   label = gtk_label_new (text[i]);
   gtk_table_attach (GTK_TABLE (table), label, 0, 1, i, i+1,
		     GTK_SHRINK, GTK_SHRINK, NO_PADDING, NO_PADDING);
   }
   /* Setting up the entries */
   gtk_table_attach (GTK_TABLE (table), start_entry, 1, 2, 0, 1,
		     GTK_FILL | GTK_SHRINK | GTK_EXPAND,
		     GTK_FILL | GTK_SHRINK | GTK_EXPAND,
		     NO_PADDING, NO_PADDING);
   gtk_table_attach (GTK_TABLE (table), stop_entry, 1, 2, 1, 2,
		     GTK_FILL | GTK_SHRINK | GTK_EXPAND,
		     GTK_FILL | GTK_SHRINK | GTK_EXPAND,
		     NO_PADDING, NO_PADDING);
   gtk_table_attach (GTK_TABLE (table), n_step_entry, 1, 2, 2, 3,
		     GTK_FILL | GTK_SHRINK | GTK_EXPAND,
		     GTK_FILL | GTK_SHRINK | GTK_EXPAND,
		     NO_PADDING, NO_PADDING);
   gtk_table_attach (GTK_TABLE (table), raccuracy_entry, 1, 2, 3, 4,
		     GTK_FILL | GTK_SHRINK | GTK_EXPAND,
		     GTK_FILL | GTK_SHRINK | GTK_EXPAND,
		     NO_PADDING, NO_PADDING);
   gtk_table_attach (GTK_TABLE (table), aaccuracy_entry, 1, 2, 4, 5,
		     GTK_FILL | GTK_SHRINK | GTK_EXPAND,
		     GTK_FILL | GTK_SHRINK | GTK_EXPAND,
		     NO_PADDING, NO_PADDING);
   gtk_signal_connect (GTK_OBJECT(start_entry), "insert_text",
		     GTK_SIGNAL_FUNC(inserted_text_handler),
		     NULL);
   gtk_signal_connect (GTK_OBJECT (start_entry), "activate",
		       GTK_SIGNAL_FUNC (changed_text_handler),
		       GINT_TO_POINTER (E_START));
   gtk_signal_connect (GTK_OBJECT(stop_entry), "insert_text",
		      GTK_SIGNAL_FUNC(inserted_text_handler),
		      NULL);
   gtk_signal_connect (GTK_OBJECT (stop_entry), "activate",
		       GTK_SIGNAL_FUNC (changed_text_handler),
		       GINT_TO_POINTER (E_STOP));
   gtk_signal_connect (GTK_OBJECT(n_step_entry), "insert_text",
		      GTK_SIGNAL_FUNC(inserted_text_handler),
		      NULL);
   gtk_signal_connect (GTK_OBJECT (n_step_entry), "activate",
		       GTK_SIGNAL_FUNC (changed_text_handler),
		       GINT_TO_POINTER (E_N_STEP));
   gtk_signal_connect (GTK_OBJECT(raccuracy_entry), "insert_text",
		      GTK_SIGNAL_FUNC(inserted_text_handler),
		      NULL);
   gtk_signal_connect (GTK_OBJECT (raccuracy_entry), "activate",
		       GTK_SIGNAL_FUNC (changed_text_handler),
		       GINT_TO_POINTER (E_RACCURACY));
   gtk_signal_connect (GTK_OBJECT(aaccuracy_entry), "insert_text",
		      GTK_SIGNAL_FUNC(inserted_text_handler),
		      NULL);
   gtk_signal_connect (GTK_OBJECT (aaccuracy_entry), "activate",
		       GTK_SIGNAL_FUNC (changed_text_handler),
		       GINT_TO_POINTER (E_AACCURACY));
   /* The measure widgets and... the compute button */
   gtk_table_attach (GTK_TABLE (table), measure_widgets->widget1,
		     2, 3, 0, 1, GTK_SHRINK, GTK_SHRINK,
		     NO_PADDING, NO_PADDING);
   gtk_table_attach (GTK_TABLE (table), measure_widgets->widget2,
		     2, 3, 1, 2, GTK_FILL | GTK_SHRINK,
		     GTK_FILL | GTK_SHRINK, NO_PADDING, NO_PADDING);
   gtk_table_attach (GTK_TABLE (table), simulate_button,
		     2, 3, 3, 4, GTK_SHRINK, GTK_SHRINK,
		     NO_PADDING, NO_PADDING);
   /* For now simulation is only newtonian */
   gtk_signal_connect (GTK_OBJECT (simulate_button), "clicked", 
		       GTK_SIGNAL_FUNC (simulate_newtonian), NULL);

   /* Pack it all */
   gtk_container_add (GTK_CONTAINER (handle_box), table);
   gtk_box_pack_start (GTK_BOX (def_vbox), handle_box,
   FALSE, FALSE, NO_PADDING);
   gtk_widget_show_all (handle_box);
}  

int main (int argc,
	  char *argv[])
{
  /* Main declarations */
  GtkWidget *main_window = NULL;
  GtkWidget *main_box = NULL;
  GtkWidget *main_toolbar = NULL; 
  GtkWidget *status_bar = NULL;
  GtkTooltips *main_tooltips = NULL;
  gint context_id;
  gint i;
  gint id;
  /* Control panel's declarations */
  GtkWidget *def_vbox = NULL;
  GtkTooltips *ct_panel_tooltips = NULL;

  /* Detect the locale specifications */
  gtk_set_locale ();

  gtk_init (&argc, &argv);

  /* Check that OpenGL is supported, if not exit */
  if (gdk_gl_query() == FALSE)
    {
      g_print("OpenGL not supported\n");
      return 0;
    }

  /* The look of the interface, load the custom file */
  //  gtk_rc_parse ("/usr/share/themes/Pixmap/gtk/gtkrc");

  /* Initialize the list of widgets and other global variables*/
  for (i = 0; i < N_WIDGETS; i++)
    {
      global_widgets[i].widget = NULL;
      global_widgets[i].exists = FALSE;
    }
  input = NULL;
  output = NULL;
  simu_parameters = NULL;
  grp_display_list = NULL;

  /* -- Main window -- */
  main_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_signal_connect (GTK_OBJECT (main_window), "delete_event",
		      GTK_SIGNAL_FUNC (gtk_main_quit), NULL);
  gtk_window_set_title (GTK_WINDOW (main_window), WINDOW_COSMIX);
  gtk_container_set_border_width (GTK_CONTAINER (main_window),
				  WINDOW_BORDER_WIDTH);
  gtk_window_set_default_size (GTK_WINDOW (main_window),
			       TOPLEVEL_HSIZE, TOPLEVEL_VSIZE);
  gtk_window_set_policy (GTK_WINDOW (main_window), TRUE, TRUE,
			 FALSE);
  gtk_widget_realize (main_window);
 
  /* Is said to be necessary in order to free the GLX contexts associated
     to the GL areas, but produces segfault (do here or at the GtkGLArea
     level ?
     gtk_quit_add_destroy(1, GTK_OBJECT(main_window)); */

  /* Initialize the tooltips */
  main_tooltips = gtk_tooltips_new ();
  gtk_tooltips_enable (main_tooltips);
  gtk_tooltips_set_delay (main_tooltips, TOOLTIPS_DELAY);
  /* Main box */
  main_box = gtk_vbox_new (FALSE, NO_PADDING);
  gtk_container_add (GTK_CONTAINER (main_window), main_box);
  gtk_widget_show (main_box); 
  /* Main menu */
  main_menu_create (main_window, main_box);
  /* Main toolbar */
  main_toolbar_create (main_window, main_box, &main_toolbar);
  /* Notebook */
  main_notebook_create (main_box, main_tooltips);
  /* Status bar */
  main_status_bar_create (main_box, &status_bar, &context_id);
  
  /* -- Control panel -- */
  id = CONTROL_PANEL;
  SETUP_WINDOW (global_widgets[id].widget,
		WINDOW_CONTROL_PANEL, GINT_TO_POINTER (id));
  gtk_window_set_policy (GTK_WINDOW (global_widgets[id].widget),
			 TRUE, TRUE, FALSE);
  gtk_widget_realize (global_widgets[id].widget);
  gtk_window_add_accel_group
    (GTK_WINDOW (global_widgets[id].widget), 
     (GtkAccelGroup *) ((gtk_accel_groups_from_object (GTK_OBJECT (main_window)))
			->data));
  
  /* Initialize the tooltips for the main window when building it 
   (ct_panel_tooltips) */
  ct_panel_tooltips = gtk_tooltips_new ();
  /* System Tree */
  def_vbox = gtk_vbox_new (FALSE, NO_PADDING);
  gtk_container_add (GTK_CONTAINER (global_widgets[id].widget),
		     def_vbox);
  tree_controls_create (global_widgets[id].widget,
		       def_vbox);
  /* Def controls */
  phy_controls_create (def_vbox);
  ki_controls_create (def_vbox);
  simu_controls_create (def_vbox);

  /* Grp controls */

  /* Tbl controls */

  /* Scr controls */

  /* Display */
  gtk_signal_connect (GTK_OBJECT (global_widgets[NOTEBOOK].widget),
		      "switch-page", GTK_SIGNAL_FUNC (control_panel_display),
		      def_vbox);
  
  gtk_widget_show (main_window);
  gtk_widget_show (global_widgets[CONTROL_PANEL].widget);
  gtk_main ();

  /* The GLib must have been compiled with --enable-mem-profile=yes for
     this to produce anu effect *
     g_mem_profile (); */

  return (0);
}
