/*  COSMIX: general n_body simulator, version 0.4 Beta
    Copyright (C) 2001 Shan Mignot

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

/* -- Macros -- */
#define SETUP_DIALOG(window, title, id) window = gtk_dialog_new (); DESTROY_POLICY(window, id); gtk_container_set_border_width (GTK_CONTAINER (window), DIALOG_BORDER_WIDTH); gtk_window_set_title (GTK_WINDOW (window), title); global_widgets[GPOINTER_TO_INT (id)].exists = TRUE;

#define SETUP_WINDOW(window, title, id) window = gtk_window_new (GTK_WINDOW_TOPLEVEL); HIDE_POLICY(window, id); gtk_container_set_border_width (GTK_CONTAINER (window), WINDOW_BORDER_WIDTH); gtk_window_set_title (GTK_WINDOW (window), title); global_widgets[GPOINTER_TO_INT (id)].exists = TRUE;

#define DESTROY_POLICY(window, id) gtk_signal_connect (GTK_OBJECT (window), "delete_event", GTK_SIGNAL_FUNC (window_destroy_event), id);

#define HIDE_POLICY(window, id) gtk_signal_connect (GTK_OBJECT (window), "delete_event", GTK_SIGNAL_FUNC (gtk_widget_hide), id);

#define ENTRY_WINDOW(window, title, id) window = gtk_window_new (GTK_WINDOW_DIALOG); DESTROY_POLICY (window, id); gtk_window_set_title (GTK_WINDOW (window), title); gtk_container_set_border_width (GTK_CONTAINER (window), WINDOW_BORDER_WIDTH); gtk_window_set_position (GTK_WINDOW (window), GTK_WIN_POS_MOUSE); 

#define SETUP_MESSAGE_DIALOG(title, txt) SETUP_ERROR_DIALOG (title, txt); g_free (txt);

#define SETUP_ERROR_DIALOG(title, txt) {GtkWidget *message = gtk_label_new (txt); GtkWidget *ok_button = gtk_button_new_with_label (OK); gint id = ERROR_DIALOG; if (global_widgets[id].exists) gtk_widget_destroy (global_widgets[id].widget); SETUP_DIALOG (global_widgets[id].widget, title, GINT_TO_POINTER (id)); global_widgets[id].exists = TRUE; gtk_box_pack_start (GTK_BOX (GTK_DIALOG (global_widgets[id].widget)-> vbox), message, FALSE, FALSE, NO_PADDING); gtk_signal_connect (GTK_OBJECT (ok_button), "clicked", GTK_SIGNAL_FUNC (window_destroy_signal), GINT_TO_POINTER (id)); gtk_box_pack_start (GTK_BOX (GTK_DIALOG (global_widgets[id].widget)-> action_area), ok_button, FALSE, FALSE, NO_PADDING); GTK_WIDGET_SET_FLAGS (ok_button, GTK_CAN_DEFAULT); gtk_widget_grab_default (ok_button); gtk_widget_show_all (global_widgets[id].widget);}

#define SETUP_LOAD_SAVE_DIALOG(title, directory, extension, callback) {gint id = PROJECT_LOAD_SAVE_DIALOG; if (global_widgets[id].exists) gtk_widget_destroy (global_widgets[id].widget); else global_widgets[id].exists = TRUE; global_widgets[id].widget = gtk_file_selection_new (title); gtk_file_selection_set_filename (GTK_FILE_SELECTION (global_widgets[id].widget), directory); gtk_file_selection_complete (GTK_FILE_SELECTION (global_widgets[id].widget), extension); gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (global_widgets[id].widget)->cancel_button), "clicked", GTK_SIGNAL_FUNC (window_destroy_signal), GINT_TO_POINTER (id)); gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION(global_widgets[id].widget)->ok_button), "clicked", GTK_SIGNAL_FUNC (callback), GTK_OBJECT (global_widgets[id].widget)); gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (global_widgets[id].widget)->ok_button), "clicked", GTK_SIGNAL_FUNC (window_destroy_signal), GINT_TO_POINTER (id)); DESTROY_POLICY(global_widgets[id].widget, GINT_TO_POINTER (id));gtk_widget_show (global_widgets[id].widget);}

/* -- Parameters -- */
/* Scientific */
#define G_CONSTANT              6.672e-11
#define WINDOW_BORDER_WIDTH             0
#define DIALOG_BORDER_WIDTH             5
#define TOPLEVEL_HSIZE                550
#define TOPLEVEL_VSIZE                588
#define TREE_WINDOW_HSIZE             250
#define TREE_WINDOW_VSIZE             300
#define ENTRY_HSIZE                    85
#define ENTRY_VSIZE                    20
#define COMBO_HSIZE                    65
#define COMBO_VSIZE                    20
#define NO_PADDING                      0
#define SPACING                         3
#define HANDLE_SIZE                    10
#define GUTTER_SIZE                     2
#define PANED_POSITION                255
#define N_TABLE_COLUMNS                11     /* 18 in reality */
#define N_WIDGETS                      21     /* The cardinal of global_widgets */
#define N_SYS_COL                       5
#define N_TREE_COL                      0
#define N_DIGITS                       10
#define DECIMAL_POSITION                1
#define TXT_LENGTH                     50
#define TOOLTIPS_DELAY                 10
#define MAX_STRING_LENGTH             100
#define FRAME_VIEW_RATIO              1.0
#define FRAME_MAX_COORD               1e3
#define FILE_EXTENSION_PROJECT     "*.prj"
#define FILE_EXTENSION_SYSTEM      "*.xml"
#define FILE_EXTENSION_RESULTS     "*.txt"

/* -- Default parameters (arbitrary for now) -- */
#define DEFAULT_N_STEP              10000
#define DEFAULT_RACCURACY            1e-9
#define DEFAULT_AACCURACY            1e-9
#define DEFAULT_OBJ_DIMENSION          10
#define DEFAULT_MAX_N_STEP        1000000
#define DEFAULT_GL_ACCURACY           0.5     /* In pixels                 */
#define DEFAULT_GL_MAX_HEIGHT         800     /* Both depend on the screen */
#define DEFAULT_GL_MAX_WIDTH         1100     /* resolution                */
#define DEFAULT_GL_MIN_FOV              5     /* Min and Max fov in        */
#define DEFAULT_GL_MAX_FOV            140     /* degrees                   */
#define DEFAULT_COMPRESSION_MODE        0     /* Compress xml files        */

/* -- Enumerated types -- */
typedef enum {MAIN_V, SUB_H1, SUB_H2} PanedWindows;
typedef enum {TOP_LEFT, TOP_RIGHT, BTM_LEFT, BTM_RIGHT} Positions;
/* DEF_GL* as well as GRP_GL* must be contiguous */
typedef enum {NOTEBOOK, SYSTEM_TREE, PROJECT_LOAD_SAVE_DIALOG, QUIT_DIALOG,
	      CONTROL_PANEL, TEXT_ENTRY, DEF_PHY, DEF_KI,
	      DEF_SIMU, ERROR_DIALOG, TBL, DEF_GL_X, DEF_GL_Y,
	      DEF_GL_Z, DEF_GL_USER, GRP_GL_X, GRP_GL_Y, GRP_GL_Z,
	      GRP_GL_USER, SCRIPT, HELP_WINDOW} GlobalWidgets;
typedef enum {E_MASS, E_DIMENSION, E_POSITION_X, E_POSITION_Y, E_POSITION_Z,
	      E_SPEED_X, E_SPEED_Y, E_SPEED_Z, E_START, E_STOP, E_N_STEP,
	      E_RACCURACY, E_AACCURACY} Entries;
typedef enum {MASS, DIMENSION} Measures;
typedef enum {OBJECT, PHY, KI} Structures;
typedef enum {PTOBJECT} Object_types;
typedef enum {X, Y, Z, T} Coordinates;
typedef enum {C_OBJ, C_T, C_X, C_Y, C_Z, C_VX, C_VY, C_VZ, C_ENERGY,
	      C_K_ENERGY, C_G_ENERGY, C_G, C_FORCE_X, C_FORCE_Y, C_FORCE_Z,
	      C_K_MOMENTUM_X, C_K_MOMENTUM_Y, C_K_MOMENTUM_Z} Table_columns;

/* -- Data structures -- */
typedef struct GtkWidgetList_decl
{
  GtkWidget         *widget;
  gboolean          exists;
} GtkWidgetList; 

typedef struct TwoWidgets_decl
{
  GtkWidget         *widget1;
  GtkWidget         *widget2;
} TwoWidgets;

typedef struct Structure_decl
{
  gint              type;
  gpointer          *structure;
} Structure;

typedef struct PhyObject_decl
{
  gint              object_type;
  gchar             *name;
  gboolean          active;
  gboolean          ok;
  gchar             *comment;
} PhyObject;

typedef struct PtObjetcPhy_decl
{
  gdouble           mass;
  gboolean          mass_ok;
  gdouble           dimension;
  gboolean          dimension_exists;
  gboolean          ok;
  gchar             *comment;
} PtObjectPhy;

typedef struct PtObjectKi_decl
{
  gdouble           positions[3];
  gboolean          position_x_ok;
  gboolean          position_y_ok;
  gboolean          position_z_ok;
  gdouble           speeds[3];
  gboolean          speed_x_ok;
  gboolean          speed_y_ok;
  gboolean          speed_z_ok;
  gboolean          ok;
  gchar             *comment;
} PtObjectKi;

typedef struct Simulation_decl
{
  gint              theory;
  gboolean          theory_ok;
  gdouble           start;
  gboolean          start_ok;
  gdouble           stop;
  gboolean          stop_ok;
  gint              n_step;
  gboolean          n_step_exists;
  gdouble           raccuracy;
  gboolean          raccuracy_exists;
  gdouble           aaccuracy;
  gboolean          aaccuracy_exists;
  gboolean          ok;
} Simulation;

/* The data passed to the integrator for the newtonian case and the results */ 
/* The input data structures */
typedef struct NControl_decl
{
  gdouble           start;
  gdouble           stop;
  gint              n_step;
  gdouble           step_size;
  gdouble           raccuracy;
  gdouble           aaccuracy;
} NControl;

typedef struct NPhyInput_decl
{
  gdouble           *masses;
  gdouble           *dimensions;
} NPhyInput;

typedef struct NKiInput_decl
{
  gdouble           *positions;
  gdouble           *speeds;
} NKiInput;
  
typedef struct Input_decl
{
  gint              n_body;
  gpointer          *controls;
  gpointer          *phy;
  gpointer          *ki;
} Input;

/* The output data structures */
typedef struct NObjectOutput_decl
{
  gchar             *name;
  gdouble           mass;
  gdouble           dimension;
  gdouble           *x;
  gdouble           *y;
  gdouble           *z;
  gdouble           *vx;
  gdouble           *vy;
  gdouble           *vz;
} NObjectOutput;

typedef struct Output_decl
{
  gint              n_body;
  gpointer          *controls;         /* Field for the control parameter */
  gpointer          *objects;          /* Field for the objects' properties */
} Output;

/* Usefull to compute the different contributions to the force */
typedef struct Quotient_decl
{
  int               i;
  int               j;
  double            quotient;
} Quotient;

/* Information needed to display OpenGl mesh */
typedef struct 
{
  gboolean          do_init;           /* TRUE if initgl not yet called       */
  gboolean          parallel;          /* TRUE if the perspective is parallel */
  float             beginx, beginy;    /* Position of mouse                   */

  //  float         quat[4];           /* Orientation of object               */
  float             fov;               /* Field of view in degrees            */
  GLdouble          lookat[9];         /* Settings to view all                */
  GLfloat           bounding_box[6];   /* Min and max in x, y,z directions    */
} mesh_info;

/* The structure containing the display lists */
typedef struct
{
  GLuint            first_list;
  GLsizei           range;
  GLuint            *display;
} GLDisplayLists;

/* -- Global variables -- */
GtkWidgetList global_widgets[N_WIDGETS];
Input *input;
Output *output;
Simulation *simu_parameters;
GLDisplayLists *grp_display_list;

