/*  COSMIX: general n_body simulator, version 0.4 Beta
    Copyright (C) 2001 Shan Mignot

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#include <stdlib.h>
#include <stdio.h>
#include <gtk/gtk.h>
#include <GL/gl.h>
#include <GL/glu.h>
#include <gtkgl/gtkglarea.h>
#include <libxml/tree.h>
#include <libxml/xmlmemory.h>
#include <libxml/parser.h>

#include "define.h"
#include "functions.h"
#include "text.h"
#include "dop853.h"

/* The XML error callback fonction. */
void xml_error (void *ctx, const char *msg)
{
  SETUP_ERROR_DIALOG(ERROR, msg);
}

/* Load a script into the script page */
void parse_script_file (GtkWidget *widget, gpointer data)
{
  char *filename;
  FILE *file;
  char c;

  if (!global_widgets[SCRIPT].exists)
    return;
  filename = 
    gtk_file_selection_get_filename (GTK_FILE_SELECTION(widget));

  gtk_text_freeze (GTK_TEXT (global_widgets[SCRIPT].widget));
  file = fopen (filename, "r");
  if (!file) SETUP_ERROR_DIALOG(ERROR, FILE_OPEN_ERROR_MESSAGE);
  script_new (NULL, NULL);
  while ((c = fgetc (file)) != EOF)
    gtk_text_insert (GTK_TEXT (global_widgets[SCRIPT].widget),
		     NULL, NULL, NULL, &c, 1);
  gtk_text_thaw (GTK_TEXT (global_widgets[SCRIPT].widget));
}

void parse_project_file (GtkWidget *widget, gpointer data)
{
}

/* Save the script as a system file */
void script_save_as_system (GtkWidget *widget, gpointer data)
{
  GtkEditable *script = NULL;
  char *filename, *text;
  FILE *file;

  filename = 
    gtk_file_selection_get_filename (GTK_FILE_SELECTION(widget));
  remove (filename); 
  file = fopen (filename, "a+"); 

  if (global_widgets[SCRIPT].exists)
    {
      script = GTK_EDITABLE (global_widgets[SCRIPT].widget);

      text = gtk_editable_get_chars (script, 0, -1);
      fprintf (file, "%s", text);
      g_free (text);
    }
	
  fclose (file);
}

/* Save the results as a text file setup like the TBL page */
void results_save_as_text (GtkWidget *widget, gpointer data)
{
  GtkCList *tbl = NULL;
  char *filename;
  char *cells[N_TABLE_COLUMNS];
  FILE *file;
  int i, k, j;

  if (!global_widgets[TBL].exists || !output)
    return; 
  else
    tbl = GTK_CLIST (global_widgets[TBL].widget);

  filename = 
    gtk_file_selection_get_filename (GTK_FILE_SELECTION(widget));
  remove (filename); 
  file = fopen (filename, "a+"); 

  for (i = 0; i < output->n_body; i++)
    {
      for (k = 0; k < ((NControl*) output->controls)->n_step; k++)
	{
	  for (j = 0; j < N_TABLE_COLUMNS; j++)
	    {
	      gtk_clist_get_text (tbl,
				  k + i*((NControl*) output->controls)->n_step,
				  j, cells + j);
	      if (strcmp (cells[j], NO_TEXT) == 0)
		cells[j] = "0";
	    }
	  	  
	  fprintf (file,
		   "%s %s %s %s %s %s %s %s %s %s %s %s %s %s %s %s %s %s\n",
		   cells[0], cells[1], cells[2], cells[3], cells[4],
		   cells[5], cells[6], cells[7], cells[8], cells[9],
		   cells[10], cells[11], cells[12], cells[13],
		   cells[14], cells[15], cells[16], cells[17]);

	  /* Is this necessary ??
	     for (j = 0; j < N_TABLE_COLUMNS; j++)
	     cells[j] = NO_TEXT; */
	}
    }

  fclose (file);
}

/* -- Save system as xml file -- */
/* Traverse the SYSTEM_TREE to build a DOM tree from scratch,
   then save it to file. Also updates the script page. */
     
void system_save_as_xml (GtkWidget *widget, gpointer data)
{

  GtkCTreeNode *node, *child;
  GtkCTree *system_tree = GTK_CTREE (global_widgets[SYSTEM_TREE].widget);
  PhyObject *object = NULL;
  PtObjectPhy *phy = NULL;
  PtObjectKi *ki = NULL;
  Structure *structure;

  xmlDocPtr doc;
  xmlNodePtr system, simu, object_node, element;
  xmlNsPtr ns;

  char value[MAX_STRING_LENGTH];
  int nbodies = 0;
  char *filename;
  char c;
  FILE *file;

  /* The root of the DOM tree */ 
  doc = xmlNewDoc("1.0");
  doc->children = xmlNewDocNode(doc, NULL, "simulation", NULL);
  ns = xmlNewNs (doc->children, "http://www.ensmp.fr/~98mignot", "sys");

  node = gtk_ctree_node_nth (system_tree, 0);

  /* The physical objects */
  if (node)
    system = xmlNewChild(doc->children, ns, "physicalsystem", NULL);
  else return;

  while (node)
    {
      /* Retrieve the object's properties */
      object_node = xmlNewChild(system, ns, "pointobject", NULL);
      structure = (Structure*) gtk_ctree_node_get_row_data (system_tree, node);
      object = (PhyObject*) structure->structure;

      /* Skip the object_type node for now */

      if (object->active == TRUE) xmlSetProp(object_node, "active", "true");
      else xmlSetProp(object_node, "active", "false");

      /* Retrieve the physical properties */
      element = xmlNewChild (object_node, ns, "pointphysics", NULL);
      child = GTK_CTREE_ROW (node)->children;
      structure = (Structure*) gtk_ctree_node_get_row_data (system_tree, child);
      phy = (PtObjectPhy*) structure->structure;
      
      if (phy->mass_ok)
	{
	  sprintf (value, "%.10e", phy->mass);
	  xmlSetProp (element, "mass", value);
	}

      if (phy->dimension_exists)
	{
	  sprintf (value, "%.10e", phy->dimension);
	  xmlSetProp (element, "dimension", value);
	}

      if (phy->comment)
	xmlNewChild (element, ns, "comment", phy->comment);

      /* Retrieve the kinetics */
      child = GTK_CTREE_ROW (child)->sibling;
      structure = (Structure*) gtk_ctree_node_get_row_data (system_tree, child);
      ki = (PtObjectKi*) structure->structure;
      element = xmlNewChild (object_node, ns, "pointkinetics", NULL);

      /* Position */
      xmlNewChild (element, ns, "pointposition", NULL);
      if (ki->position_x_ok)
	{
	  sprintf (value, "%.10e", ki->positions[X]);
	  xmlSetProp (element->children, "x", value);
	}

      if (ki->position_y_ok)
	{
	  sprintf (value, "%.10e", ki->positions[Y]);
	  xmlSetProp (element->children, "y", value);
	}

      if (ki->position_z_ok)
	{
	  sprintf (value, "%.10e", ki->positions[Z]);
	  xmlSetProp (element->children, "z", value);
	}

      /* Speed */
      xmlNewChild (element, ns, "pointspeed", NULL);
      if (ki->speed_x_ok)
	{
	  sprintf (value, "%.10e", ki->speeds[X]);
	  xmlSetProp (element->children->next, "vx", value);
	}

      if (ki->speed_y_ok)
	{
	  sprintf (value, "%.10e", ki->speeds[Y]);
	  xmlSetProp (element->children->next, "vy", value);
	}

      if (ki->speed_z_ok)
	{
	  sprintf (value, "%.10e", ki->speeds[Z]);
	  xmlSetProp (element->children->next, "vz", value);
	}
		   
      if (ki->comment)
	xmlNewChild (element, ns, "comment", ki->comment);

      if (object->name) 
	xmlNewChild (object_node, ns, "name", object->name);
      if (object->comment)
	xmlNewChild (object_node, ns, "comment", object->comment);

      nbodies++;
      node = GTK_CTREE_ROW (node)->sibling;
    }
  
  if (nbodies != 0)
    {
      sprintf (value, "%d", nbodies);
      xmlSetProp (system, "nbody", value);
    }
  
  /* The simulation parameters */
  if (simu_parameters)
    {
      simu = xmlNewChild(doc->children, ns, "newtonianparameters", NULL);

      /* Skip the theory node for now */

      if (simu_parameters->start_ok)
	{
	  sprintf (value, "%.10e", simu_parameters->start);
	  xmlSetProp (simu, "start", value);
	}

      if (simu_parameters->stop_ok)
	{
	  sprintf (value, "%.10e", simu_parameters->stop);
	  xmlSetProp (simu, "stop", value);
	}
      
      if (simu_parameters->n_step_exists)
	{
	  sprintf (value, "%d", simu_parameters->n_step);
	  xmlSetProp (simu, "nstep", value);
	}

      if (simu_parameters->raccuracy_exists)
	{
	  sprintf (value, "%.10e", simu_parameters->raccuracy);
	  xmlSetProp (simu, "raccuracy", value);
	}

      if (simu_parameters->aaccuracy_exists)
	{
	  sprintf (value, "%.10e", simu_parameters->aaccuracy);
	  xmlSetProp (simu, "aaccuracy", value);
	}
    }

  /* Write to file */
  filename = 
    gtk_file_selection_get_filename (GTK_FILE_SELECTION(widget));
  xmlSetDocCompressMode (doc, DEFAULT_COMPRESSION_MODE);
  xmlSaveFile (filename, doc);
  xmlFreeDoc (doc); 

  /* Set as script in the notebook script page, remove the previous
   script first */
  gtk_text_freeze (GTK_TEXT (global_widgets[SCRIPT].widget));
  file = fopen (filename, "r");
  if (!file) g_print ("Erreur a l'ouverture du fichier\n");
  gtk_editable_delete_text (GTK_EDITABLE (global_widgets[SCRIPT].widget), 0, -1);
  while ((c = fgetc (file)) != EOF)
    gtk_text_insert (GTK_TEXT (global_widgets[SCRIPT].widget),
		     NULL, NULL, NULL, &c, 1);
  gtk_text_thaw (GTK_TEXT (global_widgets[SCRIPT].widget));
}

/* -- Load system from an  xml file -- */
/* Buils a DOM tree from the file content, then scan it and put data in 
   the system tree. Also updates the script page. */

void parse_simulation_parameters(xmlDocPtr doc,
				 xmlNsPtr ns,
				 xmlNodePtr cur)
{
  Simulation *simu = simu_parameters;
  char* conversion_residue = NULL;
  xmlChar *data = NULL;
  gboolean invalid = FALSE;
  GList *control_panel_children = NULL;
  GtkWidget *child = NULL;

  /* Get the strings and convert them to numbers of appropriate types */
  if ((data = xmlGetProp(cur, "start")))
    {
      simu->start = strtod (data, &conversion_residue);
      simu->start_ok = TRUE;
    }
  else simu->start_ok = FALSE;

  if ((data = xmlGetProp(cur, "stop")))
    {
      simu->stop = strtod (data, &conversion_residue);
      simu->stop_ok = TRUE;
    }
  else simu->stop_ok = FALSE;

  if (simu->stop <= simu->start)
    {
      simu->start_ok = FALSE;
      simu->stop_ok = FALSE;
      invalid = TRUE;
    }
  
  if ((data = xmlGetProp(cur, "nstep")))
    {
      simu->n_step = (int) strtol (data, &conversion_residue, 0);
      simu->n_step_exists = TRUE;
      if (simu->n_step <= 0)
	{
	  simu->n_step_exists = FALSE;
	  invalid = TRUE;
	}
    }
  else simu->n_step_exists = FALSE;

  if ((data = xmlGetProp(cur, "raccuracy")))
    {
      simu->raccuracy = strtod (data, &conversion_residue);
      simu->raccuracy_exists = TRUE;
      if (simu->raccuracy <= 0) 
	{
	  simu->raccuracy_exists = FALSE;
	  invalid = TRUE;
	}
    }
  else simu->raccuracy_exists = FALSE;

  if ((data = xmlGetProp(cur, "aaccuracy")))
    {
      simu->aaccuracy = strtod (data, &conversion_residue);
      simu->aaccuracy_exists = TRUE;
      if (simu->aaccuracy <= 0)
	{
	  simu->aaccuracy_exists = FALSE;
	  invalid = TRUE;
	}
    }
  else simu->aaccuracy_exists = FALSE;
  
  /* The error messages in case some of the parameters are not of 
     valid formats or correspond to invalid values */
  if (conversion_residue)
    {
      if (strcmp (conversion_residue, "") || invalid)
	{
	  fprintf (stderr, "All values in simuparameters are not valid \
                        double format numbers.\n");
	  //      free (conversion_residue);
	}
    }

  /* Update the corresponding entries */

  if (!global_widgets[CONTROL_PANEL].exists) return;
   
  control_panel_children = gtk_container_children
    (GTK_CONTAINER (global_widgets[CONTROL_PANEL].widget));
  control_panel_children = gtk_container_children
    (GTK_CONTAINER (control_panel_children->data));
  /* Get the last child of the control panel */
  while (control_panel_children->next)
    control_panel_children = control_panel_children->next;
  control_panel_children = gtk_container_children
    (GTK_CONTAINER (control_panel_children->data));
  control_panel_children = gtk_container_children 
    (GTK_CONTAINER (control_panel_children->data))->next->next->next;

  child = GTK_WIDGET (control_panel_children->data);
  if (!simu->aaccuracy_exists ||
      !text_insert_and_check (GTK_ENTRY (child), simu->aaccuracy, TRUE, TRUE))
    gtk_entry_set_text (GTK_ENTRY (child), NO_TEXT);
  control_panel_children = control_panel_children->next;
  child = GTK_WIDGET (control_panel_children->data);
  if (!simu->raccuracy_exists ||
      !text_insert_and_check (GTK_ENTRY (child), simu->raccuracy, TRUE, TRUE))
    gtk_entry_set_text (GTK_ENTRY (child), NO_TEXT);
  control_panel_children = control_panel_children->next;
  child = GTK_WIDGET (control_panel_children->data);
  if (!simu->n_step_exists ||
      !text_insert_and_check (GTK_ENTRY (child), simu->n_step, TRUE, TRUE))
    gtk_entry_set_text (GTK_ENTRY (child), NO_TEXT);
  control_panel_children = control_panel_children->next;
  child = GTK_WIDGET (control_panel_children->data);
  if (!simu->stop_ok ||
      !text_insert_and_check (GTK_ENTRY (child), simu->stop, TRUE, TRUE))
    gtk_entry_set_text (GTK_ENTRY (child), NO_TEXT);
  control_panel_children = control_panel_children->next;
  child = GTK_WIDGET (control_panel_children->data);
  if (!simu->start_ok ||
      !text_insert_and_check (GTK_ENTRY (child), simu->start, TRUE, TRUE))
    gtk_entry_set_text (GTK_ENTRY (child), NO_TEXT);
}

/* Pass destination pointers unallocated */
void parse_point_object (xmlDocPtr doc,
			 xmlNsPtr ns,
			 xmlNodePtr cur)
{
  PhyObject *object = NULL;
  PtObjectPhy *phy = NULL;
  PtObjectKi *ki = NULL;
  xmlNodePtr child = NULL;
  char *conversion_residue;
  xmlChar *data = NULL;
  gboolean invalid = FALSE;

  /* allocate the structs */
  object = (PhyObject*) g_new0 (PhyObject, 1);
  phy = (PtObjectPhy*) g_new0 (PtObjectPhy, 1);
  ki = (PtObjectKi*) g_new0 (PtObjectKi, 1);
        
  /* Retrieve the active attribute */
  if (!strcmp (xmlGetProp (cur, "active"), "true"))
    object->active = TRUE;
  else object->active = FALSE;

  /* We don't care what the top level element name is */
  cur = cur->xmlChildrenNode;
  while (cur != NULL)
    {

      if ((!strcmp(cur->name, "pointphysics")) && (cur->ns == ns))
	{
	  /* Get mass */
	  if ((data = xmlGetProp(cur, "mass")))
	  {
	    phy->mass = strtod (data, &conversion_residue);
	    phy->mass_ok = TRUE;
	    if (phy->mass < 0) 
	      {
		phy->mass_ok = FALSE;
		invalid = TRUE;
	      }
	  }
	  else phy->mass_ok = FALSE;

	  if (phy->mass_ok) phy->ok = TRUE;
	  else phy->ok = FALSE;
			      
	  /* Get dimension */
	  if ((data = xmlGetProp (cur, "dimension")))
	    {
	      phy->dimension = strtod (data, &conversion_residue);
	      phy->dimension_exists = TRUE;
	      if (phy->dimension < 0)
		{
		  phy->dimension_exists = FALSE;
		  invalid = TRUE;
		}
	    }
	  else phy->dimension_exists = FALSE;

	  /* Get comment if it exists */
	  if ((child = cur->xmlChildrenNode) != NULL)
	    {
	      if (child->ns == ns)
		phy->comment = xmlNodeGetContent (child);
	    }

	  cur = cur->next;
	  continue;
	}

      /* No tests can be performed to check that values are valid because they can
	 be just about everything */
      if ((!strcmp(cur->name, "pointkinetics")) && (cur->ns == ns))
	{
	  child = cur->xmlChildrenNode;
	  
	  while (child)
	    {
	      /* Get position (is required) */
	      if (!strcmp (child->name, "pointposition") && (child->ns == ns))
		{
		  if ((data = xmlGetProp(child, "x")))
		    {
		      ki->positions[X] = strtod (data, &conversion_residue);
		      ki->position_x_ok = TRUE;
		    }
		  else ki->position_x_ok = FALSE;

		  if ((data = xmlGetProp(child, "y")))
		    {
		      ki->positions[Y] = strtod (data, &conversion_residue);
		      ki->position_y_ok = TRUE;
		    }
		  else ki->position_y_ok = FALSE;

		  if ((data = xmlGetProp(child, "z")))
		    {
		      ki->positions[Z] = strtod (data, &conversion_residue);
		      ki->position_z_ok = TRUE;
		    }
		  else ki->position_z_ok = FALSE;
		}
	      
	      /* Get speed (is required) */
	      if (!strcmp (child->name, "pointspeed") && (child->ns == ns))
		{
		  if ((data = xmlGetProp(child, "vx")))
		    {
		      ki->speeds[X] = strtod (data, &conversion_residue);
		      ki->speed_x_ok = TRUE;
		    }
		  else ki->speed_x_ok = FALSE;

		  if ((data = xmlGetProp(child, "vy")))
		    {
		      ki->speeds[Y] = strtod (data, &conversion_residue);
		      ki->speed_y_ok = TRUE;
		    }
		  else ki->speed_y_ok = FALSE;

		  if ((data = xmlGetProp(child, "vz")))
		    {
		      ki->speeds[Z] = strtod (data, &conversion_residue);
		      ki->speed_z_ok = TRUE;
		    }
		  else ki->speed_z_ok = FALSE;
		}
	      
	      if (ki->position_x_ok && ki->position_y_ok &&
		  ki->position_z_ok && ki->speed_x_ok && ki->speed_y_ok &&
		  ki->speed_z_ok)
		ki->ok = TRUE;

	      if (!strcmp (child->name, "comment") && (child->ns == ns))
		ki->comment = xmlNodeGetContent (child);
	      child = child->next;
	    }
	}

      if (!strcmp (cur->name, "comment") && (cur->ns == ns))
	object->comment = xmlNodeGetContent (cur);

      if (!strcmp (cur->name, "name") && (cur->ns == ns))
	object->name = xmlNodeGetContent (cur);

      cur = cur->next;
      continue;
    }

  if (strcmp (conversion_residue, "") || invalid) 
    {
      fprintf (stderr, "All values in pointobject are not valid double\
                        format numbers\n");
      //      free (conversion_residue);
    }
  
  /* Now add the object to the system tree */
  if (phy->ok && ki->ok) object->ok = TRUE;
  pt_obj_append (object, phy, ki); 
}

void parse_physicalsystem_file (GtkWidget *widget,
				gpointer data)
{
  /* Usefull only if checking against the DTD is active
     xmlValidCtxtPtr context; */

  xmlDocPtr doc;
  xmlDtdPtr dtd;
  xmlNsPtr ns;
  xmlNodePtr cur;
  xmlNodePtr child;
  char* filename = NULL;
  gint i;

  LIBXML_TEST_VERSION
    xmlKeepBlanksDefault(0);

  /* build an XML tree from the file  */
  filename = 
    gtk_file_selection_get_filename (GTK_FILE_SELECTION(widget));
  doc = xmlParseFile(filename);
  if (doc == NULL)
    {
      SETUP_ERROR_DIALOG(ERROR, INVALID_XML_MESSAGE);
      return;
    }

  /* Validate the XML file against the DTD */
  /* Pass pointer functions 
     context->error = xml_error;
     context->warning = xml_error; */

  /* Parsing the DTD segfault unless the source is compiled with
     the -O3 option, in which case it still seems to mess up the
     memory 

     dtd = xmlParseDTD (NULL, DTD_NAME);
     if (xmlValidateDtd (context, doc, dtd) == 0)
     {
     SETUP_ERROR_DIALOG(ERROR, INVALID_XML_MESSAGE);
     xmlFreeDoc (doc);
     xmlFreeDtd (dtd);
     return; 
     } */
  
  /* Check that the document is of the right kind  */
  cur = xmlDocGetRootElement(doc);
  if (cur == NULL)
    {
      SETUP_ERROR_DIALOG(ERROR, EMPTY_XML_MESSAGE);
      xmlFreeDtd (dtd);
      return;
    }
  if (strcmp(cur->name, "simulation")) 
    {
      SETUP_ERROR_DIALOG(ERROR, ROOT_XML_MESSAGE);
      xmlFreeDoc (doc);
      xmlFreeDtd (dtd);
      return;
    }
  ns = xmlSearchNsByHref(doc, cur, "http://www.ensmp.fr/~98mignot");
  if (ns == NULL)
    {
      SETUP_ERROR_DIALOG(ERROR, NAMESPACE_XML_MESSAGE);
      xmlFreeDoc (doc);
      xmlFreeDtd (dtd);
      return;
    }

  /* Now, walk the tree. First level we expect physicalsystem
     or newtonianparameters */
  cur = cur->xmlChildrenNode;
  while ( cur && xmlIsBlankNode ( cur ) )
    {
      cur = cur -> next;
    }
  if ( cur == 0 )
    return;
  if ((strcmp (cur->name, "physicalsystem") &&
       strcmp (cur->name, "newtonianparameters")) ||
      (cur->ns != ns))
    {
      SETUP_ERROR_DIALOG(ERROR, FIRST_NODE_TYPE_XML_MESSAGE);
      fprintf (stderr,"xmlDocDump follows\n");
      xmlDocDump (stderr, doc);
      fprintf (stderr,"xmlDocDump finished\n");
      xmlFreeDoc (doc);
      xmlFreeDtd (dtd);
      return;
    }

  /* The file is valid, delete the previous system */
  system_new (NULL, 1, NULL);
  
  while (cur != NULL)
    {
      if ((!strcmp (cur->name, "physicalsystem")) &&
	  (cur->ns == ns))
	{
	  /* Second level is a list of pointobject */
	  child = cur->xmlChildrenNode;
	  while (child != NULL)
	    {
	      if ((!strcmp(child->name, "pointobject")) && (cur->ns == ns))
		parse_point_object (doc, ns, child);
	      child = child->next;
	    }
	}

      if ((!strcmp (cur->name, "newtonianparameters")) &&
	  (cur->ns == ns))
	{
	  /* Second level are simulation parameters */
	  parse_simulation_parameters (doc, ns, cur);
	}

      cur = cur->next;
    }  

  /* The tree has been processed, delete now */
  xmlFreeDoc (doc);
  /* Disable the dtd functions (see higher up)
     xmlFreeDtd (dtd); */

  /* Set as script, remove the previous script first */
  parse_script_file (widget, NULL);

  /* Update the DEF_GL areas */
  def_mesh_create ();
  if (global_widgets[DEF_GL_X].exists)
    for (i = 0; i < 4; i++)
      redraw_viewport (global_widgets[DEF_GL_X + i].widget);

  /*  gtk_text_freeze (GTK_TEXT (global_widgets[SCRIPT].widget));
      file = fopen (filename, "r");
      if (!file) SETUP_ERROR_DIALOG(ERROR, FILE_OPEN_ERROR_MESSAGE);
      gtk_editable_delete_text (GTK_EDITABLE (global_widgets[SCRIPT].widget), 0, -1);
      while ((c = fgetc (file)) != EOF)
      gtk_text_insert (GTK_TEXT (global_widgets[SCRIPT].widget),
      NULL, NULL, NULL, &c, 1);
      gtk_text_thaw (GTK_TEXT (global_widgets[SCRIPT].widget)); */
}


 
