/*  COSMIX: general n_body simulator, version 0.4 Beta
    Copyright (C) 2001 Shan Mignot

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

/* -- cosmix.c -- */

int main (int argc, char *argv[]);
void main_menu_create (GtkWidget *main_window, GtkWidget *main_box);
void main_toolbar_create (GtkWidget *main_window, GtkWidget *main_box,
			  GtkWidget **main_toolbar);
GtkWidget *new_pixmap_widget (GdkWindow *window, GdkColor *background,
			      char *filename);
GdkPixmap *new_pixmap (GdkWindow *window, GdkColor *background,
		       gchar *filename);
GdkBitmap *new_bitmap (GdkWindow *window, GdkColor *background,
		       gchar *filename);
void main_notebook_create (GtkWidget *main_box,
			   GtkTooltips *main_tooltips);
void main_status_bar_create (GtkWidget *main_box, GtkWidget **status_bar,
			     gint *context_id);
void tree_controls_create (GtkWidget *control_panel_window, GtkWidget *def_vbox);
void phy_controls_create (GtkWidget *def_vbox);
void ki_controls_create (GtkWidget *def_vbox);
void simu_controls_create (GtkWidget *def_vbox);

/* -- callback.c -- */

/* Close and destroy functions */
gint window_destroy_event (GtkWidget *widget, GdkEvent *event, gpointer id);
void window_destroy_signal (GtkWidget *widget, gpointer id);
void destroy_structure (gpointer *data);
/* Main menu */
void project_new (GtkWidget *widget, gpointer data);
void system_new (gpointer data, guint callback_action, GtkWidget *widget);
void tbl_new (GtkWidget *widget, gpointer data);
void script_new (GtkWidget *widget, gpointer data);
void project_load (GtkWidget *widget, gpointer data);
void system_load (GtkWidget *widget, gpointer data);
void script_load (GtkWidget *widget, gpointer data);
void objects_load (GtkWidget *widget, gpointer data);
void cnsts_load (GtkWidget *widget, gpointer data);
void reload (GtkWidget *widget, gpointer data);
void reset (GtkWidget *widget, gpointer data);
void save (GtkWidget *widget, gpointer data);
void project_save_as (GtkWidget *widget, gpointer data);
void system_save_as (GtkWidget *widget, gpointer data);
void script_save_as (GtkWidget *widget, gpointer data);
void tbl_save_as (GtkWidget *widget, gpointer data);
void animation_save_as (GtkWidget *widget, gpointer data);
void objects_save_as (GtkWidget *widget, gpointer data);
void cnsts_save_as (GtkWidget *widget, gpointer data);
void properties (GtkWidget *widget, gpointer data);
void quit (GtkWidget *widget, gpointer data);
void window_show (GtkWidget *widget, gpointer data);
void getting_started (GtkWidget *widget, gpointer data);
void about (GtkWidget *widget, gpointer data);
/* Control Panel */
void tree_utils (GtkCList *system_list, gint column, gpointer data);
void edit_name (GtkWidget *widget, gpointer *data);
void edit_comment (GtkWidget *widget, gpointer *data);
gint tree_popup_menu (GtkWidget *widget, GdkEventButton *event, gpointer data);
void expand_tree (gpointer data, guint callback_action, GtkWidget *widget);
void copy_element (gpointer data, guint callback_action, GtkWidget *widget);
void delete_element (gpointer data, guint callback_action, GtkWidget *widget);
void pt_obj_new (GtkWidget *widget, gpointer data);
void pt_obj_append (PhyObject *object, PtObjectPhy *phy, PtObjectKi *ki);
/* void display_selected_row (GtkObject* object, gint row, gint column,
   GdkEventButton *event, gpointer data); */
void display_selected_row (GtkObject *object, gint row, gint column,
			   GdkEventButton *event, gpointer data);
void control_panel_display (GtkNotebook *notebook, GtkNotebookPage *page,
			    gint page_num, gpointer user_data);
void measure_mode (GtkWidget *widget, gpointer *data);
void update_measure (gpointer *data, guint callback_action,
		     GtkWidget *widget);
void inserted_text_handler (GtkEntry *entry, const gchar *text, gint length,
			    gint *position, gpointer data);
void changed_text_handler (GtkEntry *entry, gpointer data);
gboolean text_insert_and_check (GtkEntry *entry, gdouble value, gboolean positive,
				gboolean zero);
void tbl_hide_column  (GtkCList *list, gint index, gpointer data);
void simulate_newtonian (GtkButton *simulate_button, gpointer data);
void grp_fill (void);
void tbl_fill (void);

/* -- grp.c -- */
void glu_error (GLenum errorCode);
void redraw_viewport (GtkWidget *widget);
void initgl (void);
gint glarea_expose (GtkWidget *widget, GdkEventExpose *event);
gint glarea_configure (GtkWidget *widget, GdkEventConfigure *event);
gint glarea_destroy (GtkWidget *widget);
gint glarea_button_press (GtkWidget *widget, GdkEventButton *event);
gint glarea_motion_notify_perspective (GtkWidget *widget, GdkEventMotion *event);
gint glarea_motion_notify_parallel (GtkWidget *widget, GdkEventMotion *event);
void gl_area_set (GtkWidget *frame, GlobalWidgets i);
void def_mesh_create (void);
void render_def_mesh (void);
void grp_mesh_create (void);
void render_grp_mesh (void);


/* -- fileio.c -- */
void xml_error (void *ctx, const char *msg);
void parse_script_file (GtkWidget *widget, gpointer data);
void parse_project_file (GtkWidget *widget, gpointer data);
void script_save_as_system (GtkWidget *widget, gpointer data);
void results_save_as_text (GtkWidget *widget, gpointer data);
void system_save_as_xml (GtkWidget *widget, gpointer data);
void parse_simulation_parameters (xmlDocPtr doc, xmlNsPtr ns, xmlNodePtr cur);
void parse_point_object (xmlDocPtr doc, xmlNsPtr ns, xmlNodePtr cur);
void parse_point_object (xmlDocPtr doc, xmlNsPtr ns, xmlNodePtr cur);
void parse_physicalsystem_file (GtkWidget *widget, gpointer data);

